@@ -9,13 +9,8 @@
 #include "kconv.h"
 #include "func.h"
 #include "kanji.h"
-
-#ifdef	DEP_ORIGSHELL
 #include "system.h"
-#endif
-#ifdef	DEP_PTY
 #include "termemu.h"
-#endif
 
 #ifndef	_NOARCHIVE
 
@@ -23,6 +18,9 @@
 extern char *unixgetcurdir __P_((char *, int));
 #endif
 
+#if	FD >= 3
+extern int widedigit;
+#endif
 extern int mark;
 #if	FD >= 2
 extern int sorttype;
@@ -116,7 +114,9 @@ static VOID NEAR Xwaitmes __P_((VOID_A));
 static VOID NEAR unpackerror __P_((VOID_A));
 static char *archfgets __P_((VOID_P));
 static int NEAR readarchive __P_((CONST char *, lsparse_t *, int));
-static char *NEAR searcharcdir __P_((CONST char *, int));
+static CONST char *NEAR skiparchdir __P_((CONST char *));
+static int NEAR getarchdirlen __P_((CONST char *));
+static char *NEAR searcharchdir __P_((CONST char *, int));
 static char *NEAR archoutdir __P_((VOID_A));
 static int NEAR undertmp __P_((CONST char *));
 #ifdef	DEP_PSEUDOPATH
@@ -208,11 +208,13 @@ launchlist_t launchlist
 	{"*.tar.Z",	"zcat %C|tar tvf -",	PM_TAR, 0},
 	{"*.tar.gz",	"gzip -cd %C|tar tvf -",	PM_TAR, 0},
 	{"*.tar.bz2",	"bzip2 -cd %C|tar tvf -",	PM_TAR, 0},
+	{"*.tar.xz",	"xz -cd %C|tar tvf -",	PM_TAR, 0},
 # ifndef	CYGWIN
 	{"*.taZ",	"zcat %C|tar tvf -",	PM_TAR, 0},
 # endif
 	{"*.taz",	"gzip -cd %C|tar tvf -",	PM_TAR, 0},
 	{"*.tgz",	"gzip -cd %C|tar tvf -",	PM_TAR, 0},
+	{"*.txz",	"xz -cd %C|tar tvf -",	PM_TAR, 0},
 #endif	/* !MSDOS */
 	{NULL,		NULL,			PM_NULL, 0}
 };
@@ -248,6 +250,8 @@ archivelist_t archivelist
 					"gzip -cd %C|tar xf - %TA", 0},
 	{"*.tar.bz2",	"tar cf - %T|bzip2 -c > %C",
 					"bzip2 -cd %C|tar xf - %TA", 0},
+	{"*.tar.xz",	"tar cf - %T|xz -c > %C",
+					"xz -cd %C|tar xf - %TA", 0},
 # ifndef	CYGWIN
 	{"*.taZ",	"tar cf - %T|compress -c > %C",
 					"zcat %C|tar xf - %TA", 0},
@@ -256,6 +260,8 @@ archivelist_t archivelist
 					"gzip -cd %C|tar xf - %TA", 0},
 	{"*.tgz",	"tar cf - %T|gzip -c > %C",
 					"gzip -cd %C|tar xf - %TA", 0},
+	{"*.txz",	"tar cf - %T|xz -c > %C",
+					"xz -cd %C|tar xf - %TA", 0},
 #endif	/* !MSDOS */
 	{NULL,		NULL,			NULL, 0}
 };
@@ -328,20 +334,35 @@ static VOID NEAR pusharchdupl(VOID_A)
 #ifndef	_NOTREE
 	new -> v_treepath = treepath;
 #endif
-	new -> v_fullpath = NULL;
+	new -> v_fullpath =
+	new -> v_lastfile = NULL;
 	new -> v_findpattern = findpattern;
 	new -> v_filepos = filepos;
 	new -> v_sorton = sorton;
 	archduplp = new;
 }
 
-VOID escapearch(VOID_A)
+/*ARGSUSED*/
+VOID escapearch(all)
+int all;
 {
+	for (;;) {
 #ifndef	_NOBROWSE
-	if (!archduplp) return;
-	if (browselist) popbrowsevar();
+		if (!archduplp) {
+			freebrowse(browselist);
+			Xfree(archivefile);
+			browselist = NULL;
+			archivefile = NULL;
+			break;
+		}
+		if (browselist) popbrowsevar();
 #endif
-	poparchdupl();
+		poparchdupl();
+#ifndef	_NOBROWSE
+		if (all && browselist) continue;
+#endif
+		break;
+	}
 }
 
 static VOID NEAR poparchdupl(VOID_A)
@@ -420,6 +441,7 @@ CONST char *file, *dir;
 #ifndef	_NOBROWSE
 	int i;
 #endif
+	CONST char *cp;
 	char *arch;
 	int len;
 
@@ -454,27 +476,24 @@ CONST char *file, *dir;
 		Xputterm(L_CLEAR);
 
 		Xlocate(TC_PATH, TL_PATH);
-		Xputterm(T_STANDOUT);
 # ifndef	_NOBROWSE
 		if (browselist) {
-			XXcputs(TS_BROWSE);
+			cp = TS_BROWSE;
 			len = TD_BROWSE;
 		}
 		else
 # endif
 		{
-			XXcputs(TS_ARCH);
+			cp = TS_ARCH;
 			len = TD_ARCH;
 		}
-		Xputterm(END_STANDOUT);
+		Xattrputs(cp, 1);
 		cputstr(len, arch);
 		Xfree(arch);
 
 		Xlocate(TC_MARK, TL_PATH);
 		VOID_C XXcprintf("%<*d", TD_MARK, mark);
-		Xputterm(T_STANDOUT);
-		XXcputs(TS_MARK);
-		Xputterm(END_STANDOUT);
+		Xattrputs(TS_MARK, 1);
 		VOID_C XXcprintf("%<'*qd", TD_SIZE, marksize);
 
 		Xtflush();
@@ -486,19 +505,18 @@ CONST char *file, *dir;
 	Xputterm(L_CLEAR);
 
 	Xlocate(C_PATH, L_PATH);
-	Xputterm(T_STANDOUT);
 #ifndef	_NOBROWSE
 	if (browselist) {
-		XXcputs(S_BROWSE);
+		cp = S_BROWSE;
 		len = D_BROWSE;
 	}
 	else
 #endif
 	{
-		XXcputs(S_ARCH);
+		cp = S_ARCH;
 		len = D_ARCH;
 	}
-	Xputterm(END_STANDOUT);
+	Xattrputs(cp, 1);
 	cputstr(len, arch);
 	Xfree(arch);
 
@@ -573,38 +591,63 @@ int flags;
 	return(0);
 }
 
-VOID copyarcf(re, arcre)
-CONST reg_t *re;
-CONST char *arcre;
+static CONST char *NEAR skiparchdir(file)
+CONST char *file;
 {
-	char *cp, *tmp;
+	int len;
+
+	if (!*archivedir) len = 0;
+	else if ((len = dirmatchlen(archivedir, file))) file += len;
+	else return(NULL);
+
+	if (*file && len > 0 && (len > 1 || *archivedir != _SC_)) file++;
+
+	return(file);
+}
+
+static int NEAR getarchdirlen(file)
+CONST char *file;
+{
+	CONST char *tmp;
+	int len;
+
+	if (!(tmp = strdelim(file, 0))) len = strlen(file);
+	else {
+		len = (tmp == file) ? 1 : tmp - file;
+		while (*(++tmp) == _SC_) /*EMPTY*/;
+	}
+
+	if (len == 2 && file[0] == '.' && file[1] == '.') len = 0;
+	if (tmp && *tmp) return(-1 - len);
+
+	return(len);
+}
+
+VOID copyarcf(re, archre)
+CONST reg_ex_t *re;
+CONST char *archre;
+{
+	CONST char *cp;
 	int i, j, n, len, parent;
 
 	parent = (*archivedir) ? 0 : -1;
 	/* omit arcflist[0] as pseudo ".." */
 	for (i = 1; i < maxarcf; i++) {
-		if (!*archivedir) len = 0;
-		else if (!(len = dirmatchlen(archivedir, arcflist[i].name)))
-			continue;
-
-		cp = &(arcflist[i].name[len]);
-		if (*cp && len > 0 && (len > 1 || *archivedir != _SC_)) cp++;
+		if (!(cp = skiparchdir(arcflist[i].name))) continue;
 		if (!*cp) {
 			parent = i;
 			continue;
 		}
 
-		if (!(tmp = strdelim(cp, 0))) len = strlen(cp);
-		else {
-			len = (tmp == cp) ? 1 : tmp - cp;
-			while (*(++tmp) == _SC_);
+		len = getarchdirlen(cp);
+		if (len >= -1 && len <= 0) {
+			if (parent <= 0) parent = i;
 		}
+		if (len < 0) continue;
+		if (re && !regexp_exec(re, cp, 1)) continue;
 
-		if (parent <= 0 && len == 2 && cp[0] == '.' && cp[1] == '.')
-			parent = i;
-		if ((tmp && *tmp) || (re && !regexp_exec(re, cp, 1))) continue;
-		if (arcre) {
-			if (!(n = searcharc(arcre, arcflist, maxarcf, i)))
+		if (archre) {
+			if (!(n = searcharcf(archre, arcflist, maxarcf, i)))
 				continue;
 			if (n < 0) break;
 		}
@@ -616,7 +659,7 @@ CONST char *arcre;
 		filelist = addlist(filelist, maxfile, &maxent);
 		memcpy((char *)&(filelist[maxfile]),
 			(char *)&(arcflist[i]), sizeof(namelist));
-		filelist[maxfile].name = cp;
+		filelist[maxfile].name = (char *)cp;
 #ifndef	NOSYMLINK
 		filelist[maxfile].linkname = arcflist[i].linkname;
 #endif
@@ -637,41 +680,33 @@ CONST char *arcre;
 #endif
 }
 
-static char *NEAR searcharcdir(file, flen)
+static char *NEAR searcharchdir(file, flen)
 CONST char *file;
 int flen;
 {
-	char *cp, *tmp;
+	CONST char *cp;
 	int i, len;
 
 	errno = ENOENT;
-	for (i = 0; i < maxarcf; i++) {
-		if (!*archivedir) len = 0;
-		else if (!(len = dirmatchlen(archivedir, arcflist[i].name)))
-			continue;
+	/* omit arcflist[0] as pseudo ".." */
+	for (i = 1; i < maxarcf; i++) {
+		if (!(cp = skiparchdir(arcflist[i].name))) continue;
+		if (!file) return(arcflist[(*cp) ? 0 : i].name);
+		if (!*cp) continue;
 
-		cp = &(arcflist[i].name[len]);
-		if (*cp && len > 0 && (len > 1 || *archivedir != _SC_)) cp++;
-		if (!*cp) {
-			if (file) continue;
+		len = getarchdirlen(cp);
+		if (len >= -1 && len <= 0) {
+			if (*file) continue;
+			return(arcflist[0].name);
 		}
-		else {
-			if (!file) continue;
-			if (!(tmp = strdelim(cp, 0))) len = strlen(cp);
-			else {
-				len = (tmp == cp) ? 1 : tmp - cp;
-				while (*(++tmp) == _SC_);
-			}
-			if (!*file && len == 2 && cp[0] == '.' && cp[1] == '.')
-				return(arcflist[0].name);
-			if ((tmp && *tmp) || len != flen
-			|| strnpathcmp(file, cp, flen))
-				continue;
-			if (!isdir(&(arcflist[i]))) {
-				errno = ENOTDIR;
-				continue;
-			}
+		if (len < 0) continue;
+		if (len != flen || strnpathcmp(file, cp, flen)) continue;
+
+		if (!isdir(&(arcflist[i]))) {
+			errno = ENOTDIR;
+			continue;
 		}
+
 		return(arcflist[i].name);
 	}
 
@@ -683,14 +718,14 @@ static char *NEAR archoutdir(VOID_A)
 	char *cp, *file;
 
 	if (!*archivedir) return((char *)-1);
-	if (!(file = searcharcdir(NULL, 0))) return(NULL);
+	if (!(file = searcharchdir(NULL, 0))) return(NULL);
 
 	cp = &(file[(int)strlen(file) - 1]);
 	while (cp > file && *cp == _SC_) cp--;
 #ifdef	BSPATHDELIM
 	if (onkanji1(file, cp - file)) cp++;
 #endif
-	cp = strrdelim2(file, cp);
+	cp = strrdelim2(file, 0, cp);
 	if (!cp) *archivedir = '\0';
 	else {
 		if (cp == file) copyrootpath(archivedir);
@@ -708,7 +743,7 @@ CONST char *path;
 	int i, n, flags, dupfilepos;
 #endif
 	CONST char *cp, *file;
-	char *tmp, duparcdir[MAXPATHLEN];
+	char *tmp, duparchdir[MAXPATHLEN];
 	int len;
 
 	Xfree(findpattern);
@@ -722,8 +757,8 @@ CONST char *path;
 				errno = ENOENT;
 				return(NULL);
 			}
-			Xstrncpy(duparcdir, path, (cp - path));
-			path = duparcdir;
+			Xstrncpy(duparchdir, path, (cp - path));
+			path = duparchdir;
 		}
 
 		n = browselevel;
@@ -763,21 +798,26 @@ CONST char *path;
 #endif	/* !_NOBROWSE */
 
 	if (!path) return(archoutdir());
-	Xstrcpy(duparcdir, archivedir);
+	Xstrcpy(duparchdir, archivedir);
+	file = nullstr;
 	do {
 		if (*path == _SC_) len = 1;
 		else if ((cp = strdelim(path, 0))) len = cp - path;
 		else len = strlen(path);
 
 		cp = path;
-		if (len == 2 && path[0] == '.' && path[1] == '.') cp = nullstr;
-		if (searcharcdir(cp, len)) {
+		if (*cp != '.') /*EMPTY*/;
+		else if (len == 1) cp = curpath;
+		else if (len == 2 && cp[1] == '.') cp = nullstr;
+
+		if (searcharchdir(cp, len)) {
 			if (*(tmp = archivedir)) tmp = strcatdelim(archivedir);
 			Xstrncpy(tmp, path, len);
 			file = parentpath;
 		}
+		else if (cp == curpath) /*EMPTY*/;
 		else if (*cp || !(file = archoutdir())) {
-			Xstrcpy(archivedir, duparcdir);
+			Xstrcpy(archivedir, duparchdir);
 			errno = ENOENT;
 			return(NULL);
 		}
@@ -797,14 +837,14 @@ int flen, argc;
 char ***argvp;
 {
 	CONST char *cp, *file;
-	char *tmp, *new, dir[MAXPATHLEN], duparcdir[MAXPATHLEN];
+	char *new, dir[MAXPATHLEN], duparchdir[MAXPATHLEN];
 	int i, len, parent;
 
 # ifdef	DEP_DOSPATH
 	if (_dospath(path)) return(argc);
 # endif
 
-	Xstrcpy(duparcdir, archivedir);
+	Xstrcpy(duparchdir, archivedir);
 	if (!(file = strrdelim(path, 0))) file = path;
 # ifndef	_NOBROWSE
 	else if (browselist) return(argc);
@@ -812,7 +852,7 @@ char ***argvp;
 	else {
 		Xstrncpy(dir, path, (file == path) ? 1 : file - path);
 		if (!(cp = archchdir(dir)) || cp == (char *)-1) {
-			Xstrcpy(archivedir, duparcdir);
+			Xstrcpy(archivedir, duparchdir);
 			return(argc);
 		}
 		flen -= ++file - path;
@@ -820,24 +860,17 @@ char ***argvp;
 
 	/* omit arcflist[0] as pseudo ".." */
 	for (i = 1; i < maxarcf; i++) {
-		if (!*archivedir) len = 0;
-		else if (!(len = dirmatchlen(archivedir, arcflist[i].name)))
-			continue;
-
-		cp = &(arcflist[i].name[len]);
-		if (*cp && len > 0 && (len > 1 || *archivedir != _SC_)) cp++;
+		if (!(cp = skiparchdir(arcflist[i].name))) continue;
 		if (!*cp) continue;
 
-		if (!(tmp = strdelim(cp, 0))) len = strlen(cp);
-		else {
-			len = (tmp == cp) ? 1 : tmp - cp;
-			while (*(++tmp) == _SC_);
-		}
-
 		parent = 0;
-		if (len == 2 && cp[0] == '.' && cp[1] == '.') parent++;
-		else if (tmp && *tmp) continue;
-		if (strnpathcmp(file, cp, flen)) continue;
+		len = getarchdirlen(cp);
+		if (len >= -1 && len <= 0) {
+			len = 2;
+			parent++;
+		}
+		if (len < 0) continue;
+		if (len != flen || strnpathcmp(file, cp, flen)) continue;
 
 		new = Xmalloc(len + 1 + 1);
 		Xstrncpy(new, cp, len);
@@ -845,7 +878,7 @@ char ***argvp;
 		new[len] = '\0';
 		argc = addcompletion(NULL, new, argc, argvp);
 	}
-	Xstrcpy(archivedir, duparcdir);
+	Xstrcpy(archivedir, duparchdir);
 
 	return(argc);
 }
@@ -934,8 +967,7 @@ int flags;
 		poparchdupl();
 		return(-1);
 	}
-	maxfile = 0;
-	filepos = -1;
+	maxfile = filepos = 0;
 
 	return(1);
 }
@@ -946,7 +978,7 @@ int launcher(VOID_A)
 	CONST char *cp;
 	int flags;
 #endif
-	reg_t *re;
+	reg_ex_t *re;
 	int i, n;
 
 #ifndef	_NOBROWSE
@@ -996,10 +1028,10 @@ CONST char *path;
 	char *full, rpath[MAXPATHLEN], dir[MAXPATHLEN];
 
 	full = fullpath;
-	Xrealpath(path, rpath, RLP_READLINK);
+	VOID_C Xrealpath(path, rpath, RLP_READLINK);
 	for (wvp = archduplp; wvp; wvp = wvp -> v_archduplp) {
 		if (!(wvp -> v_fullpath)) continue;
-		Xrealpath(full, dir, RLP_READLINK);
+		VOID_C Xrealpath(full, dir, RLP_READLINK);
 		if (underpath(rpath, dir, -1)) return(1);
 		full = wvp -> v_fullpath;
 	}
@@ -1092,7 +1124,7 @@ CONST char *full;
 # endif
 	if (full) Xstrcpy(fullpath, full);
 	else if (*cp != _SC_) return(-1);
-	Xrealpath(path, path, RLP_READLINK);
+	VOID_C Xrealpath(path, path, RLP_READLINK);
 	Xstrcpy(fullpath, dupfullpath);
 
 	if (!(cp = dostmpdir(drive))) return(-1);
@@ -1102,15 +1134,15 @@ CONST char *full;
 }
 #endif	/* DEP_PSEUDOPATH */
 
-int pack(arc)
-CONST char *arc;
+int pack(arch)
+CONST char *arch;
 {
 #ifdef	DEP_PSEUDOPATH
 	char *dest, *tmpdest;
 	int drive;
 #endif
 	winvartable *wvp;
-	reg_t *re;
+	reg_ex_t *re;
 	char *tmpdir, *full, *duparchivefile, path[MAXPATHLEN];
 	int i, n, ret;
 
@@ -1120,7 +1152,7 @@ CONST char *arc;
 		if (archivelist[i].flags & AF_IGNORECASE) pathignorecase++;
 #endif
 		re = regexp_init(archivelist[i].ext, -1);
-		n = regexp_exec(re, arc, 0);
+		n = regexp_exec(re, arch, 0);
 		regexp_free(re);
 #ifndef	PATHNOCASE
 		if (archivelist[i].flags & AF_IGNORECASE) pathignorecase--;
@@ -1139,11 +1171,11 @@ CONST char *arc;
 	tmpdest = NULL;
 	drive = 0;
 
-	Xstrcpy(path, arc);
+	Xstrcpy(path, arch);
 	if ((dest = strrdelim(path, 1))) *(++dest) = '\0';
 	else copycurpath(path);
 	if ((n = archdostmpdir(path, &tmpdest, full)) < 0) {
-		warning(ENOENT, arc);
+		warning(ENOENT, arch);
 		return(0);
 	}
 	if (n) dest = Xstrdup(path);
@@ -1165,9 +1197,9 @@ CONST char *arc;
 #endif
 
 #ifdef	DEP_PSEUDOPATH
-	if (!genfullpath(path, arc, full, tmpdest)) warning(-1, arc);
+	if (!genfullpath(path, arch, full, tmpdest)) warning(-1, arch);
 #else
-	if (!genfullpath(path, arc, full)) warning(-1, arc);
+	if (!genfullpath(path, arch, full)) warning(-1, arch);
 #endif
 	else {
 		duparchivefile = archivefile;
@@ -1202,8 +1234,8 @@ CONST char *arc;
 }
 
 /*ARGSUSED*/
-int unpack(arc, dir, arg, tr, flags)
-CONST char *arc, *dir, *arg;
+int unpack(arch, dir, arg, tr, flags)
+CONST char *arch, *dir, *arg;
 int tr, flags;
 {
 #ifdef	DEP_PSEUDOPATH
@@ -1212,7 +1244,7 @@ int tr, flags;
 	char *full, *dest, *tmpdest;
 	int drive, dupmaxfile, dupfilepos;
 #endif
-	reg_t *re;
+	reg_ex_t *re;
 	char *cp, *new, *tmpdir, path[MAXPATHLEN];
 	int i, n, ret;
 
@@ -1228,7 +1260,7 @@ int tr, flags;
 		if (archivelist[i].flags & AF_IGNORECASE) pathignorecase++;
 #endif
 		re = regexp_init(archivelist[i].ext, -1);
-		n = regexp_exec(re, arc, 0);
+		n = regexp_exec(re, arch, 0);
 		regexp_free(re);
 #ifndef	PATHNOCASE
 		if (archivelist[i].flags & AF_IGNORECASE) pathignorecase--;
@@ -1291,7 +1323,7 @@ int tr, flags;
 	dupfilelist = filelist;
 	dupmaxfile = maxfile;
 	dupfilepos = filepos;
-	alist[0].name = (char *)arc;
+	alist[0].name = (char *)arch;
 # ifndef	NOSYMLINK
 	alist[0].linkname = NULL;
 # endif
@@ -1299,20 +1331,20 @@ int tr, flags;
 	filelist = alist;
 	maxfile = 1;
 	filepos = 0;
-	drive = tmpdosdupl(arc, &tmpdir, 0);
+	drive = tmpdosdupl(arch, &tmpdir, 0);
 	filelist = dupfilelist;
 	maxfile = dupmaxfile;
 	filepos = dupfilepos;
 
-	if (drive < 0) warning(-1, arc);
+	if (drive < 0) warning(-1, arch);
 	else
 #endif	/* DEP_PSEUDOPATH */
 	if ((!dir && preparedir(path) < 0) || _chdir2(cp) < 0)
 		warning(-1, path);
 #ifdef	DEP_PSEUDOPATH
-	else if (!genfullpath(path, arc, fullpath, tmpdir)) warning(-1, arc);
+	else if (!genfullpath(path, arch, fullpath, tmpdir)) warning(-1, arch);
 #else
-	else if (!genfullpath(path, arc, fullpath)) warning(-1, arc);
+	else if (!genfullpath(path, arch, fullpath)) warning(-1, arch);
 #endif
 	else {
 		Xwaitmes();
@@ -1332,7 +1364,7 @@ int tr, flags;
 		Xfree(dest);
 		removetmp(tmpdest, NULL);
 	}
-	if (tmpdir) removetmp(tmpdir, arc);
+	if (tmpdir) removetmp(tmpdir, arch);
 	else
 #endif	/* DEP_PSEUDOPATH */
 	if (_chdir2(fullpath) < 0) lostcwd(fullpath);
@@ -1371,29 +1403,29 @@ namelist *list;
 CONST char *dir;
 {
 	namelist duplist;
-	char *cp, path[MAXPATHLEN], duparcdir[MAXPATHLEN];
+	char *cp, path[MAXPATHLEN], duparchdir[MAXPATHLEN];
 	int i, ret;
 
 	if (!(list -> linkname)) return(0);
 
 	if (!islink(list)) cp = list -> linkname;
 	else {
-		strcatdelim2(duparcdir, archivedir, list -> linkname);
-		cp = duparcdir;
+		strcatdelim2(duparchdir, archivedir, list -> linkname);
+		cp = duparchdir;
 	}
 	*path = '\0';
 	if (archrealpath(cp, path) < 0) return(-1);
 
 	for (i = 0; i < maxarcf; i++) {
-		*duparcdir = '\0';
-		if (archrealpath(arcflist[i].name, duparcdir) < 0) continue;
-		if (!strcmp(path, duparcdir)) break;
+		*duparchdir = '\0';
+		if (archrealpath(arcflist[i].name, duparchdir) < 0) continue;
+		if (!strcmp(path, duparchdir)) break;
 	}
 	if (i >= maxarcf) return(-1);
 
 	if (unpacklink(&(arcflist[i]), dir) < 0) return(-1);
 
-	Xstrcpy(duparcdir, archivedir);
+	Xstrcpy(duparchdir, archivedir);
 	memcpy(&duplist, &(filelist[filepos]), sizeof(namelist));
 	memcpy(&(filelist[filepos]), &(arcflist[i]), sizeof(namelist));
 	if (!(cp = strrdelim(filelist[filepos].name, 0))) *archivedir = '\0';
@@ -1405,7 +1437,7 @@ CONST char *dir;
 	ret = unpack(archivefile, dir, NULL,
 		0, F_ARGSET | F_ISARCH | F_NOADDOPT);
 	memcpy(&(filelist[filepos]), &duplist, sizeof(namelist));
-	Xstrcpy(archivedir, duparcdir);
+	Xstrcpy(archivedir, duparchdir);
 
 	return(ret);
 }
@@ -1529,7 +1561,7 @@ CONST char *dev;
 	return(1);
 }
 
-int searcharc(regstr, flist, maxf, n)
+int searcharcf(regstr, flist, maxf, n)
 CONST char *regstr;
 namelist *flist;
 int maxf, n;
@@ -1537,10 +1569,10 @@ int maxf, n;
 	namelist tmp;
 	lsparse_t *list;
 	namelist *dupfilelist;
-	reg_t *re;
+	reg_ex_t *re;
 	XFILE *fp;
 	char *cp, *next, *tmpdir, *file;
-	int i, c, no, match, dupmaxfile, dupfilepos;
+	int i, c, nline, match, dupmaxfile, dupfilepos;
 
 	if (n < 0) {
 		file = flist -> name;
@@ -1593,9 +1625,7 @@ int maxf, n;
 
 	Xlocate(0, L_MESLINE);
 	Xputterm(L_CLEAR);
-	Xputterm(T_STANDOUT);
-	VOID_C Xkanjiputs(SEACH_K);
-	Xputterm(END_STANDOUT);
+	VOID_C Xattrkanjiputs(SEACH_K, 1);
 	VOID_C Xkanjiputs(file);
 	Xtflush();
 
@@ -1604,12 +1634,12 @@ int maxf, n;
 		Xfree(cp);
 	}
 
-	no = match = 0;
+	nline = match = 0;
 	re = regexp_init(regstr, -1);
 	parsefilelist(NULL, list, NULL, NULL, NULL);
 	for (;;) {
-		if (match && match <= no - (int)(list -> bottomskip)) break;
-		i = parsefilelist(fp, list, &tmp, &no, archfgets);
+		if (match && match <= nline - (int)(list -> bottomskip)) break;
+		i = parsefilelist(fp, list, &tmp, &nline, archfgets);
 		if (i < 0) {
 			if (i == -1) break;
 			VOID_C pclose2(fp);
@@ -1626,7 +1656,7 @@ int maxf, n;
 			if (regexp_exec(re, cp, 1)) break;
 			cp = next;
 		}
-		if (cp) match = no;
+		if (cp) match = nline;
 		Xfree(tmp.name);
 #ifndef	NOSYMLINK
 		Xfree(tmp.linkname);
@@ -1637,7 +1667,7 @@ int maxf, n;
 	if (n >= 0) removetmp(tmpdir, file);
 
 	if ((i = pclose2(fp))) return((i > 0) ? -1 : 0);
-	if (match > no - (int)(list -> bottomskip)) match = 0;
+	if (match > nline - (int)(list -> bottomskip)) match = 0;
 
 	return(match);
 }
