@@ -9,13 +9,8 @@
 #include "func.h"
 #include "funcno.h"
 #include "kanji.h"
-
-#ifdef	DEP_ORIGSHELL
 #include "system.h"
-#endif
-#ifdef	DEP_PTY
 #include "termemu.h"
-#endif
 
 #if	defined (_NOEXTRAATTR) || defined (NOUID)
 #define	MAXATTRSEL		2
@@ -28,6 +23,9 @@
 #endif
 
 extern int curcolumns;
+#if	FD >= 3
+extern int widedigit;
+#endif
 extern int mark;
 extern off_t marksize;
 extern off_t blocksize;
@@ -53,7 +51,7 @@ extern int hideclock;
 extern int fdmode;
 #endif
 
-static VOID NEAR replacefname __P_((char *));
+static int NEAR dochdir4 __P_((CONST char *, int));
 static int cur_up __P_((CONST char *));
 static int cur_down __P_((CONST char *));
 static int cur_right __P_((CONST char *));
@@ -74,7 +72,7 @@ static int mark_file2 __P_((CONST char *));
 static int mark_file3 __P_((CONST char *));
 static int mark_all __P_((CONST char *));
 static int mark_reverse __P_((CONST char *));
-static reg_t *NEAR prepareregexp __P_((CONST char *, CONST char *));
+static reg_ex_t *NEAR prepareregexp __P_((CONST char *, CONST char *));
 static int mark_find __P_((CONST char *));
 static int in_dir __P_((CONST char *));
 static int out_dir __P_((CONST char *));
@@ -161,7 +159,7 @@ static int no_operation __P_((CONST char *));
 
 #include "functabl.h"
 
-reg_t *findregexp = NULL;
+reg_ex_t *findregexp = NULL;
 #ifndef	_NOWRITEFS
 int writefs = 0;
 #endif
@@ -328,15 +326,16 @@ bindlist_t bindlist
 };
 
 
-static VOID NEAR replacefname(name)
-char *name;
+static int NEAR dochdir4(path, raw)
+CONST char *path;
+int raw;
 {
-	if (filepos < maxfile) Xfree(filelist[filepos].name);
-	else for (maxfile = 0; maxfile < filepos; maxfile++)
-		if (!filelist[maxfile].name)
-			filelist[maxfile].name = Xstrdup(nullstr);
-	filelist[filepos].name = (name) ? name : Xstrdup(parentpath);
-	filelist[filepos].tmpflags |= F_ISCHGDIR;
+	if (chdir4(path, raw, archivefile) < 0) {
+		warning(-1, path);
+		return(FNC_CANCEL);
+	}
+
+	return(FNC_EFFECT);
 }
 
 static int cur_up(arg)
@@ -498,7 +497,7 @@ CONST char *arg;
 static int cur_top(arg)
 CONST char *arg;
 {
-	if (filepos == 0) return(FNC_NONE);
+	if (filepos <= 0) return(FNC_NONE);
 	filepos = 0;
 
 	return(FNC_UPDATE);
@@ -508,7 +507,7 @@ CONST char *arg;
 static int cur_bottom(arg)
 CONST char *arg;
 {
-	if (filepos == maxfile - 1) return(FNC_NONE);
+	if (filepos >= maxfile - 1) return(FNC_NONE);
 	filepos = maxfile - 1;
 
 	return(FNC_UPDATE);
@@ -713,10 +712,10 @@ CONST char *arg;
 	return(FNC_UPDATE);
 }
 
-static reg_t *NEAR prepareregexp(mes, arg)
+static reg_ex_t *NEAR prepareregexp(mes, arg)
 CONST char *mes, *arg;
 {
-	reg_t *re;
+	reg_ex_t *re;
 	char *wild;
 
 	if (arg && *arg) wild = Xstrdup(arg);
@@ -736,7 +735,7 @@ CONST char *mes, *arg;
 static int mark_find(arg)
 CONST char *arg;
 {
-	reg_t *re;
+	reg_ex_t *re;
 	int i;
 
 	if (!(re = prepareregexp(FINDF_K, arg))) return(FNC_CANCEL);
@@ -757,28 +756,29 @@ CONST char *arg;
 static int in_dir(arg)
 CONST char *arg;
 {
+	if (!isdir(&(filelist[filepos]))) return(warning_bell(arg));
 #ifndef	_NOARCHIVE
-	if (archivefile) /*EMPTY*/;
-	else
+	else if (archivefile) /*EMPTY*/;
 #endif
-	if (!isdir(&(filelist[filepos]))
-	|| isdotdir(filelist[filepos].name) == 2)
+	else if (isdotdir(filelist[filepos].name) == 2)
 		return(warning_bell(arg));
 
-	return(FNC_CHDIR);
+	return(dochdir4(filelist[filepos].name, 1));
 }
 
 /*ARGSUSED*/
 static int out_dir(arg)
 CONST char *arg;
 {
+	CONST char *path;
+
 #ifndef	_NOARCHIVE
-	if (archivefile) filepos = -1;
+	if (archivefile) path = NULL;
 	else
 #endif
-	replacefname(NULL);
+	path = parentpath;
 
-	return(FNC_CHDIR);
+	return(dochdir4(path, 1));
 }
 
 /*ARGSUSED*/
@@ -857,11 +857,8 @@ CONST char *arg;
 static int log_dir(arg)
 CONST char *arg;
 {
-#ifndef	_NOARCHIVE
-	CONST char *cp;
-	char dupfullpath[MAXPATHLEN];
-#endif
 	char *path;
+	int no;
 
 	if (arg && *arg) path = Xstrdup(arg);
 	else if (!(path = inputstr(LOGD_K, 0, -1, NULL, HST_PATH)))
@@ -870,57 +867,18 @@ CONST char *arg;
 		Xfree(path);
 		return(FNC_CANCEL);
 	}
-#ifndef	_NOARCHIVE
-	if (archivefile && *path != '/') {
-		if (!(cp = archchdir(path))) {
-			warning(-1, path);
-			Xfree(path);
-			return(FNC_CANCEL);
-		}
-		Xfree(path);
-		if (cp != (char *)-1) filelist[filepos].name = (char *)cp;
-		else escapearch();
-		return(FNC_EFFECT);
-	}
-	else
-#endif
-	if (chdir3(path, 0) < 0) {
-		warning(-1, path);
-		Xfree(path);
-		return(FNC_CANCEL);
-	}
-	Xfree(path);
-#ifndef	_NOARCHIVE
-	if (archivefile) {
-		Xstrcpy(dupfullpath, fullpath);
-		while (archivefile) {
-# ifdef	_NOBROWSE
-			escapearch();
-# else
-			do {
-				escapearch();
-			} while (browselist);
-# endif
-		}
-		Xstrcpy(fullpath, dupfullpath);
-	}
-#endif
-	replacefname(NULL);
 
-	return(FNC_EFFECT);
+	no = dochdir4(path, 0);
+	Xfree(path);
+
+	return(no);
 }
 
 /*ARGSUSED*/
 static int log_top(arg)
 CONST char *arg;
 {
-	char *path;
-
-	path = Xstrdup(rootpath);
-	if (chdir3(path, 1) < 0) error(path);
-	replacefname(path);
-
-	return(FNC_EFFECT);
+	return(dochdir4(rootpath, 1));
 }
 
 static VOID NEAR clearscreen(VOID_A)
@@ -947,7 +905,7 @@ CONST char *file;
 {
 	XFILE *fp;
 	char *buf, *prompt;
-	int i, len, min, max;
+	int i, len, siz, min, max;
 
 	if (!(fp = Xfopen(file, "r"))) {
 		warning(-1, file);
@@ -969,15 +927,16 @@ CONST char *file;
 	i = strlen3(file);
 	len = strlen2(buf);
 	if (i + len > n_lastcolumn) i = n_lastcolumn - len;
-	prompt = Xmalloc(i * KANAWID + strlen(buf) + 1);
+	siz = i * KANAWID + strlen(buf);
+	prompt = Xmalloc(siz + 1);
 	VOID_C strncpy2(prompt, file, &i, 0);
-	strcat(prompt, buf);
+	VOID_C Xstrlcat(prompt, buf, siz);
 
 	i = min;
 	while ((buf = Xfgets(fp))) {
 		Xlocate(0, i);
 		Xputterm(L_CLEAR);
-		VOID_C XXcprintf("%.*k", n_column, buf);
+		VOID_C XXcprintf("%^.*k", n_column, buf);
 		Xfree(buf);
 		if (++i >= max) {
 			i = min;
@@ -1163,8 +1122,8 @@ CONST char *arg;
 static int sort_dir(arg)
 CONST char *arg;
 {
-	CONST char *str[6];
-	int i, tmp1, tmp2, val[6], *dupl;
+	CONST char *str[MAXSORTTYPE + 1];
+	int i, n, tmp1, tmp2, val[MAXSORTTYPE + 1], *dupl;
 
 	str[0] = ONAME_K;
 	str[1] = OEXT_K;
@@ -1172,29 +1131,30 @@ CONST char *arg;
 	str[3] = ODATE_K;
 	str[4] = OLEN_K;
 	str[5] = ORAW_K;
-	val[0] = 1;
-	val[1] = 2;
-	val[2] = 3;
-	val[3] = 4;
-	val[4] = 5;
+	val[0] = SRT_FILENAME;
+	val[1] = SRT_EXTENSION;
+	val[2] = SRT_SIZE;
+	val[3] = SRT_DATE;
+	val[4] = SRT_LENGTH;
 	val[5] = 0;
 
-	tmp1 = sorton & 7;
-	tmp2 = sorton & ~7;
+	tmp1 = (sorton & SRT_TYPE);
+	tmp2 = (sorton & SRT_DESC);
 	if (arg && *arg) {
-		i = Xatoi(arg);
-		if ((i >= 0 && i <= 13) || (i & 7) <= 5) {
-			tmp1 = i & 7;
-			tmp2 = i & ~7;
+		n = Xatoi(arg);
+		if ((n >= 0 && n <= (MAXSORTTYPE | SRT_DESC))
+		|| (n & SRT_TYPE) <= MAXSORTTYPE) {
+			tmp1 = (n & SRT_TYPE);
+			tmp2 = (n & SRT_DESC);
 		}
 	}
 	else {
-		if (tmp1) i = 6;
-		else {
-			i = 5;
+		n = arraysize(str);
+		if (!tmp1) {
+			n--;
 			tmp1 = val[0];
 		}
-		if (selectstr(&tmp1, i, 0, str, val) != K_CR)
+		if (selectstr(&tmp1, n, 0, str, val) != K_CR)
 			return(FNC_CANCEL);
 	}
 
@@ -1206,10 +1166,10 @@ CONST char *arg;
 		str[0] = OINC_K;
 		str[1] = ODEC_K;
 		val[0] = 0;
-		val[1] = 8;
+		val[1] = SRT_DESC;
 		if (selectstr(&tmp2, 2, 56, str, val) != K_CR)
 			return(FNC_CANCEL);
-		sorton = tmp1 + tmp2;
+		sorton = (tmp1 | tmp2);
 		dupl = (int *)Xmalloc(maxfile * sizeof(int));
 		for (i = 0; i < maxfile; i++) {
 			dupl[i] = filelist[i].ent;
@@ -1431,7 +1391,7 @@ CONST char *arg;
 		Xfree(file);
 		file = Xstrdup(parentpath);
 	}
-	replacefname(file);
+	setlastfile(file);
 
 	return(FNC_EFFECT);
 }
@@ -1500,7 +1460,7 @@ CONST char *arg;
 {
 	char *wild;
 
-	if (arg && *arg) wild = Xstrdup(arg);
+	if (arg) wild = Xstrdup(arg);
 	else if (!(wild = inputstr(FINDF_K, 0, 0, "*", -1)))
 		return(FNC_CANCEL);
 
@@ -1540,10 +1500,13 @@ CONST char *arg;
 	if (!(tmp = strrdelim(destpath, 0))) tmp = destpath;
 	else {
 		*(tmp++) = '\0';
-		chdir3(destpath, 1);
+		if (dochdir4(destpath, 1) == FNC_CANCEL) {
+			Xfree(destpath);
+			return(FNC_CANCEL);
+		}
 	}
 
-	replacefname(Xstrdup(tmp));
+	setlastfile(tmp);
 	Xfree(destpath);
 
 	return(FNC_EFFECT);
@@ -1573,7 +1536,9 @@ CONST char *arg;
 		if (!com) return(FNC_CANCEL);
 	}
 	if (*com) {
-		ret = ptyusercomm(com, filelist[filepos].name, F_ARGSET);
+		ret = ptyusercomm(com,
+			(filepos < maxfile) ? filelist[filepos].name : NULL,
+			F_ARGSET);
 		ret = evalstatus(ret);
 	}
 	else {
@@ -1786,12 +1751,10 @@ CONST char *s;
 	n = ATR_MODEONLY;
 	Xlocate(0, ATTR_Y);
 	Xputterm(L_CLEAR);
-#if	!defined (_NOEXTRAATTR) && !defined (NOUID)
-	Xputterm(T_STANDOUT);
-#endif
+#if	defined (_NOEXTRAATTR) || defined (NOUID)
 	VOID_C Xkanjiputs(s);
-#if	!defined (_NOEXTRAATTR) && !defined (NOUID)
-	Xputterm(END_STANDOUT);
+#else
+	VOID_C Xattrkanjiputs(s, 1);
 #endif
 	if (selectstr(&n, MAXATTRSEL, ATTR_X, str, val) != K_CR) return(-1);
 
@@ -1862,19 +1825,14 @@ static int tree_dir(arg)
 CONST char *arg;
 {
 	char *path;
+	int no;
 
 	if (!(path = tree(0, NULL))) return(FNC_UPDATE);
-	if (chdir3(path, 1) < 0) {
-		warning(-1, path);
-		Xfree(path);
-		return(FNC_UPDATE);
-	}
+	no = dochdir4(path, 1);
+	if (no == FNC_CANCEL) no = FNC_UPDATE;
 	Xfree(path);
-	Xfree(findpattern);
-	findpattern = NULL;
-	replacefname(NULL);
 
-	return(FNC_EFFECT);
+	return(no);
 }
 #endif	/* !_NOTREE */
 
@@ -1942,10 +1900,8 @@ CONST char *arg;
 	isearch = 1;
 	Xlocate(0, L_HELP);
 	Xputterm(L_CLEAR);
-	Xputterm(T_STANDOUT);
-	win_x = Xkanjiputs(SEAF_K);
+	win_x = Xattrkanjiputs(SEAF_K, 1);
 	win_y = L_HELP;
-	Xputterm(END_STANDOUT);
 
 	return(FNC_NONE);
 }
@@ -1957,10 +1913,8 @@ CONST char *arg;
 	isearch = -1;
 	Xlocate(0, L_HELP);
 	Xputterm(L_CLEAR);
-	Xputterm(T_STANDOUT);
-	win_x = Xkanjiputs(SEAB_K);
+	win_x = Xattrkanjiputs(SEAB_K, 1);
 	win_y = L_HELP;
-	Xputterm(END_STANDOUT);
 
 	return(FNC_NONE);
 }
@@ -1981,6 +1935,7 @@ int oldwin;
 	memcpy((char *)filelist, (char *)(winvar[oldwin].v_filelist), i);
 	for (i = 0; i < winvar[oldwin].v_maxfile; i++)
 		filelist[i].name = Xstrdup(winvar[oldwin].v_filelist[i].name);
+	lastfile = Xstrdup(winvar[oldwin].v_lastfile);
 	filepos = winvar[oldwin].v_filepos;
 	sorton = winvar[oldwin].v_sorton;
 	dispmode = winvar[oldwin].v_dispmode;
@@ -2005,6 +1960,8 @@ int nextwin(VOID_A)
 	int oldwin;
 
 	if (windows <= 1) return(-1);
+
+	if (lastfile || internal_status == FNC_EFFECT) getfilelist();
 	oldwin = win;
 	if (++win >= windows) win = 0;
 	duplwin(oldwin);
@@ -2046,7 +2003,7 @@ CONST char *arg;
 # ifdef	_NOEXTRAWIN
 	else if (fileperrow(windows + 1) < WFILEMIN)
 # else
-	else if (winvar[win].v_fileperrow < WFILEMIN * 2 + 1)
+	else if (FILEPERROW < WFILEMIN * 2 + 1)
 # endif
 	{
 		warning(0, NOROW_K);
@@ -2065,8 +2022,7 @@ CONST char *arg;
 		calcwin();
 # else
 		i = (winvar[oldwin].v_fileperrow - 1) / 2;
-		winvar[win].v_fileperrow =
-			(winvar[oldwin].v_fileperrow - 1) - i;
+		FILEPERROW = (winvar[oldwin].v_fileperrow - 1) - i;
 		winvar[oldwin].v_fileperrow = i;
 # endif
 
