@@ -19,10 +19,8 @@
 
 #include "dirent.h"
 #include "sysemu.h"
-
-#ifdef	DEP_ORIGSHELL
 #include "system.h"
-#endif
+
 #if	defined (FD) || defined (WITHNETWORK)
 #include "realpath.h"
 #endif
@@ -110,18 +108,18 @@ static int cc_intr = -1;
 #ifndef	FD
 # if	MSDOS
 typedef struct _statfs_t {
-	u_short f_type __attribute__ ((packed));
-	u_short f_version __attribute__ ((packed));
-	u_long f_clustsize __attribute__ ((packed));
-	u_long f_sectsize __attribute__ ((packed));
-	u_long f_bavail __attribute__ ((packed));
-	u_long f_blocks __attribute__ ((packed));
-	u_long f_real_bavail_sect __attribute__ ((packed));
-	u_long f_real_blocks_sect __attribute__ ((packed));
-	u_long f_real_bavail __attribute__ ((packed));
-	u_long f_real_blocks __attribute__ ((packed));
-	u_char reserved[8] __attribute__ ((packed));
-} statfs_t;
+	u_short f_type;
+	u_short f_version;
+	u_long f_clustsize;
+	u_long f_sectsize;
+	u_long f_bavail;
+	u_long f_blocks;
+	u_long f_real_bavail_sect;
+	u_long f_real_blocks_sect;
+	u_long f_real_bavail;
+	u_long f_real_blocks;
+	u_char reserved[8];
+} X_attr_packed statfs_t;
 # else	/* !MSDOS */
 #  ifdef	USESTATVFSH
 #include <sys/statvfs.h>
@@ -216,7 +214,7 @@ static int NEAR cpsymlink __P_((CONST char *, CONST char *));
 
 static VOID NEAR doserror __P_((CONST char *, int));
 static VOID NEAR dosperror __P_((CONST char *));
-static VOID NEAR fputsize __P_((off_t *, off_t *));
+static VOID NEAR putsize __P_((off_t *, off_t *));
 static int NEAR inputkey __P_((VOID_A));
 static int cmpdirent __P_((CONST VOID_P, CONST VOID_P));
 static int NEAR filterattr __P_((CONST struct filestat_t *));
@@ -242,11 +240,11 @@ static int NEAR _checkline __P_((int));
 static VOID NEAR dosdirheader __P_((VOID_A));
 #ifdef	MINIMUMSHELL
 static VOID NEAR dosdirfooter __P_((off_t *, int, int, off_t *, off_t *));
-static int NEAR dosdir __P_((reg_t *, off_t *, int *, int *, off_t *));
+static int NEAR dosdir __P_((reg_ex_t *, off_t *, int *, int *, off_t *));
 #else
 static VOID NEAR dosdirfooter __P_((off_t *, int, int,
 		off_t *, off_t *, off_t *, off_t *));
-static int NEAR dosdir __P_((reg_t *, off_t *, int *, int *,
+static int NEAR dosdir __P_((reg_ex_t *, off_t *, int *, int *,
 		off_t *, off_t *));
 #endif
 int doscomdir __P_((int, char *CONST []));
@@ -459,7 +457,7 @@ int on;
 		tty.sg_flags |= ECHO;
 	}
 #  endif	/* !USETERMIOS && !USETERMIO */
-	tioctl(ttyio, REQSETP, &tty);
+	VOID_C tioctl(ttyio, REQSETP, &tty);
 }
 
 /*ARGSUSED*/
@@ -537,8 +535,7 @@ int n;
 		kanjifputs(s, Xstderr);
 		VOID_C fputnl(Xstderr);
 	}
-	if (doserrstr[n]) Xfputs(doserrstr[n], Xstderr);
-	VOID_C fputnl(Xstderr);
+	errputs(doserrstr[n], 1);
 }
 
 static VOID NEAR dosperror(s)
@@ -552,12 +549,11 @@ CONST char *s;
 		kanjifputs(s, Xstderr);
 		VOID_C fputnl(Xstderr);
 	}
-	Xfputs(Xstrerror(duperrno), Xstderr);
-	VOID_C fputnl(Xstderr);
+	errputs(Xstrerror(duperrno), 1);
 	errno = 0;
 }
 
-static VOID NEAR fputsize(np, bsizep)
+static VOID NEAR putsize(np, bsizep)
 off_t *np, *bsizep;
 {
 	off_t s, mb;
@@ -583,8 +579,8 @@ static int NEAR inputkey(VOID_A)
 	stdiomode();
 	if (c == EOF) c = -1;
 	else if (c == cc_intr) {
-		Xfputs("^C", Xstdout);
-		VOID_C fputnl(Xstdout);
+		puts2("^C");
+		putnl();
 		c = -1;
 	}
 
@@ -711,8 +707,7 @@ int (NEAR *getoptcmd)__P_((int, char *CONST []));
 		er++;
 		doserror(argv[n], ER_TOOMANYPARAM);
 	}
-	if (er) Xfputs("(Error occurred in environment variable)\n\n",
-		Xstdout);
+	if (er) puts2("(Error occurred in environment variable)\n\n");
 	freevar(argv);
 }
 
@@ -899,14 +894,14 @@ int verbose;
 		}
 	}
 	showstr(buf, 8, (dirflag & DF_LOWER));
-	Xfputc(' ', Xstdout);
+	putc2(' ');
 
-	if (!ext) Xfputs("   ", Xstdout);
+	if (!ext) puts2("   ");
 	else {
 		Xstrcpy(buf, ext);
 		showstr(buf, 3, (dirflag & DF_LOWER));
 	}
-	Xfputc(' ', Xstdout);
+	putc2(' ');
 
 	switch (dirp -> mod & S_IFMT) {
 		case S_IFREG:
@@ -916,17 +911,17 @@ int verbose;
 			VOID_C Xprintf("%'13qd", dirp -> siz);
 			break;
 		case S_IFDIR:
-			Xfputs("  <DIR>      ", Xstdout);
+			puts2("  <DIR>      ");
 			break;
 		default:
-			Xfputs("             ", Xstdout);
+			puts2("             ");
 			break;
 	}
 
 #ifndef	MINIMUMSHELL
 	if (verbose > 0) {
 		if ((dirp -> mod & S_IFMT) != S_IFREG)
-			Xfputs("              ", Xstdout);
+			puts2("              ");
 		else {
 			dirp -> siz = ((dirp -> siz + *bsizep - 1) / *bsizep)
 				* *bsizep;
@@ -946,7 +941,7 @@ int verbose;
 #else
 	tm = localtime(&(dirp -> mtim));
 #endif
-	Xfputs("  ", Xstdout);
+	puts2("  ");
 #ifndef	MINIMUMSHELL
 	if (verbose < 0) VOID_C Xprintf("%04d-%02d-%02d  %2d:%02d ",
 		tm -> tm_year + 1900, tm -> tm_mon + 1, tm -> tm_mday,
@@ -979,7 +974,7 @@ int verbose;
 		if ((dirp -> mod & S_IFMT) == S_IFIFO) buf[3] = 'L';
 		if (dir_isdir(dirp)) buf[4] = 'D';
 		if (dirp -> mod & S_ISVTX) buf[5] = 'A';
-		Xfputs(buf, Xstdout);
+		puts2(buf);
 	}
 #endif	/* !MINIMUMSHELL */
 
@@ -987,7 +982,7 @@ int verbose;
 #ifndef	NOSYMLINK
 	if (dirp -> lnam) VOID_C Xprintf(" -> %s", dirp -> lnam);
 #endif
-	VOID_C fputnl(Xstdout);
+	putnl();
 }
 
 static VOID NEAR showfnamew(dirp)
@@ -996,7 +991,7 @@ struct filestat_t *dirp;
 	char *ext, buf[MAXNAMLEN + 1];
 	int i, len;
 
-	if (dir_isdir(dirp)) Xfputc('[', Xstdout);
+	if (dir_isdir(dirp)) putc2('[');
 	Xstrcpy(buf, dirp -> d_alias);
 	if (isdotdir(buf)) ext = NULL;
 	else if ((ext = Xstrrchr(buf, '.'))) {
@@ -1013,14 +1008,14 @@ struct filestat_t *dirp;
 	len = showstr(buf, -8, (dirflag & DF_LOWER));
 
 	if (ext) {
-		Xfputc('.', Xstdout);
+		putc2('.');
 		len++;
 		Xstrcpy(buf, ext);
 		len += showstr(buf, -3, (dirflag & DF_LOWER));
 	}
 
-	if (dir_isdir(dirp)) Xfputc(']', Xstdout);
-	for (; len < 8 + 1 + 3; len++) Xfputc(' ', Xstdout);
+	if (dir_isdir(dirp)) putc2(']');
+	for (; len < 8 + 1 + 3; len++) putc2(' ');
 }
 
 static VOID NEAR showfnameb(dirp)
@@ -1053,8 +1048,8 @@ int n_incline;
 	if (!(dirflag & DF_PAUSE)) return(0);
 	if (dirflag & DF_CANCEL) return(-1);
 	if ((dirline += n_incline) >= n_line - 2) {
-		Xfputs("Press any key to continue . . .", Xstdout);
-		VOID_C fputnl(Xstdout);
+		puts2("Press any key to continue . . .");
+		putnl();
 		if (inputkey() < 0) {
 			dirflag |= DF_CANCEL;
 			return(-1);
@@ -1071,27 +1066,26 @@ static VOID NEAR dosdirheader(VOID_A)
 	if (dirtype != 'B') {
 		if (checkline(1) < 0) return;
 		if (dirflag & DF_SUBDIR) {
-			VOID_C fputnl(Xstdout);
+			putnl();
 			if (checkline(1) < 0) return;
 		}
-		else Xfputc(' ', Xstdout);
+		else putc2(' ');
 		VOID_C Xprintf("Directory of %K\n", dirwd);
 	}
 #ifndef	MINIMUMSHELL
 	if (dirtype == 'V') {
 		if (checkline(1) < 0) return;
-		Xfputs("File Name         Size        Allocated      ",
-			Xstdout);
-		Xfputs("Modified      Accessed  Attrib\n", Xstdout);
+		puts2("File Name         Size        Allocated      ");
+		puts2("Modified      Accessed  Attrib\n");
 		if (checkline(1) < 0) return;
-		VOID_C fputnl(Xstdout);
+		putnl();
 		if (checkline(1) < 0) return;
-		VOID_C fputnl(Xstdout);
+		putnl();
 	}
 #endif	/* !MINIMUMSHELL */
 	if (dirtype != 'B') {
 		if (checkline(1) < 0) return;
-		VOID_C fputnl(Xstdout);
+		putnl();
 	}
 }
 
@@ -1111,15 +1105,15 @@ off_t *sump, *bsump, *fp, *tp;
 	if (!nf && !nd) {
 		if (!fp) return;
 		if (checkline(1) < 0) return;
-		Xfputs("File not found\n", Xstdout);
+		puts2("File not found\n");
 		nd = -1;
 	}
 	else {
 		if (fp && (dirflag & DF_SUBDIR)) {
 			if (checkline(1) < 0) return;
-			VOID_C fputnl(Xstdout);
+			putnl();
 			if (checkline(1) < 0) return;
-			Xfputs("Total files listed:\n", Xstdout);
+			puts2("Total files listed:\n");
 		}
 		if (checkline(1) < 0) return;
 		VOID_C Xprintf("%10d file(s)%'15qd bytes\n", nf, *sump);
@@ -1127,8 +1121,8 @@ off_t *sump, *bsump, *fp, *tp;
 		if (dirtype == 'V') {
 			if (checkline(1) < 0) return;
 			VOID_C Xprintf("%10d dir(s) ", nd);
-			fputsize(bsump, bsizep);
-			Xfputs(" allocated\n", Xstdout);
+			putsize(bsump, bsizep);
+			puts2(" allocated\n");
 			nd = -1;
 		}
 #endif	/* !MINIMUMSHELL */
@@ -1136,17 +1130,17 @@ off_t *sump, *bsump, *fp, *tp;
 	if (!fp) return;
 
 	if (checkline(1) < 0) return;
-	if (nd < 0) Xfputs("                  ", Xstdout);
+	if (nd < 0) puts2("                  ");
 	else VOID_C Xprintf("%10d dir(s) ", nd);
 
-	fputsize(fp, bsizep);
-	Xfputs(" free\n", Xstdout);
+	putsize(fp, bsizep);
+	puts2(" free\n");
 
 #ifndef	MINIMUMSHELL
 	if (tp && dirtype == 'V') {
 		if (checkline(1) < 0) return;
-		Xfputs("                  ", Xstdout);
-		fputsize(tp, bsizep);
+		puts2("                  ");
+		putsize(tp, bsizep);
 		VOID_C Xprintf(" total disk space, %3d%% in use\n",
 			(int)(((*tp - *fp) * (off_t)100) / *tp));
 	}
@@ -1155,13 +1149,13 @@ off_t *sump, *bsump, *fp, *tp;
 
 #ifdef	MINIMUMSHELL
 static int NEAR dosdir(re, bsizep, nfp, ndp, sump)
-reg_t *re;
+reg_ex_t *re;
 off_t *bsizep;
 int *nfp, *ndp;
 off_t *sump;
 #else
 static int NEAR dosdir(re, bsizep, nfp, ndp, sump, bsump)
-reg_t *re;
+reg_ex_t *re;
 off_t *bsizep;
 int *nfp, *ndp;
 off_t *sump, *bsump;
@@ -1193,7 +1187,7 @@ off_t *sump, *bsump;
 	file = strcatdelim(dirwd);
 	dirlist = NULL;
 	while ((dp = Xreaddir(dirp))) {
-		Xstrcpy(file, dp -> d_name);
+		if (strcatpath(dirwd, file, dp -> d_name) < 0) continue;
 #ifndef	NOSYMLINK
 		if (Xlstat(dirwd, &st) < 0 || (st.st_mode & S_IFMT) != S_IFLNK)
 			llen = -1;
@@ -1258,8 +1252,8 @@ off_t *sump, *bsump;
 				if (!(c % 5) && checkline(1) < 0) break;
 				showfnamew(&(dirlist[n]));
 				if (c != (nf + nd - 1) && (c % 5) != 5 - 1)
-					Xfputc('\t', Xstdout);
-				else VOID_C fputnl(Xstdout);
+					putc2('\t');
+				else putnl();
 				break;
 			case 'B':
 				if (isdotdir(dirlist[n].nam)) break;
@@ -1338,7 +1332,7 @@ char *CONST argv[];
 #ifdef	DEP_PSEUDOPATH
 	int drv;
 #endif
-	reg_t *re;
+	reg_ex_t *re;
 	struct stat st;
 	CONST char *cp, *dir, *file;
 	char *tmp, wd[MAXPATHLEN], cwd[MAXPATHLEN], buf[MAXPATHLEN];
@@ -1380,7 +1374,7 @@ char *CONST argv[];
 		}
 #ifdef	DEP_URLPATH
 		else if (checkdrv(drv, NULL) == DEV_URL)
-			Xrealpath(buf, buf, 0);
+			VOID_C Xrealpath(buf, buf, 0);
 #endif
 		else {
 			dir = curpath;
@@ -1399,7 +1393,8 @@ char *CONST argv[];
 	if (dir != buf) Xstrcpy(wd, cwd);
 	else if ((cp = getrealpath(buf, wd, cwd)) == wd) /*EMPTY*/;
 #ifdef	DOUBLESLASH
-	else if (cp == buf && isdslash(buf)) Xrealpath(buf, wd, RLP_READLINK);
+	else if (cp == buf && isdslash(buf))
+		VOID_C Xrealpath(buf, wd, RLP_READLINK);
 #endif
 	else {
 #ifdef	DEP_PSEUDOPATH
@@ -1410,7 +1405,7 @@ char *CONST argv[];
 	}
 	if (getinfofs(wd, &total, &fre, &bsize) < 0) total = fre = (off_t)-1;
 
-	if (dirtype != 'B') VOID_C fputnl(Xstdout);
+	if (dirtype != 'B') putnl();
 
 	dirline = nf = nd = 0;
 #ifndef	MINIMUMSHELL
@@ -1439,7 +1434,7 @@ char *CONST argv[];
 	dosdirfooter(&bsize, nf, nd, &sum, &bsum, &fre, &total);
 #endif
 
-	Xfflush(Xstdout);
+	VOID_C Xfflush(Xstdout);
 
 	return(RET_SUCCESS);
 }
@@ -1523,9 +1518,8 @@ char *CONST argv[];
 	else buf = argv[n];
 	if (!flag && (!strcmp(buf, "*") || !strcmp(buf, "*.*"))) {
 		do {
-			Xfputs("All files in directory will be deleted!",
-				Xstdout);
-			VOID_C fputnl(Xstdout);
+			puts2("All files in directory will be deleted!");
+			putnl();
 #ifdef	FD
 			ttyiomode(1);
 			buf = inputstr("Are you sure (Y/N)?", 0, 0, NULL, -1);
@@ -1534,7 +1528,7 @@ char *CONST argv[];
 			buf = gets2("Are you sure (Y/N)?");
 #endif
 			if (!buf) return(RET_SUCCESS);
-			if (!isatty(STDIN_FILENO)) VOID_C fputnl(Xstdout);
+			if (!isatty(STDIN_FILENO)) putnl();
 			c = *buf;
 			Xfree(buf);
 		} while (!Xstrchr("ynYN", c));
@@ -1550,14 +1544,14 @@ char *CONST argv[];
 			do {
 				VOID_C Xprintf("%K,    Delete (Y/N)?",
 					wild[i]);
-				Xfflush(Xstdout);
+				VOID_C Xfflush(Xstdout);
 				if ((c = inputkey()) < 0) {
 					freevar(wild);
 					return(ret);
 				}
 				if (c <= (int)MAXUTYPE(u_char) && Xisprint(c))
-					Xfputc(c, Xstdout);
-				VOID_C fputnl(Xstdout);
+					putc2(c);
+				putnl();
 			} while (!Xstrchr("ynYN", c));
 			if (c == 'n' || c == 'N') continue;
 		}
@@ -1731,7 +1725,7 @@ CONST char *file, *src;
 
 	if (c) {
 		VOID_C Xfprintf(Xstderr, "Overwrite %K (Yes/No/All)?", file);
-		Xfflush(Xstderr);
+		VOID_C Xfflush(Xstderr);
 		key = -1;
 		for (;;) {
 			if ((c = inputkey()) < 0) {
@@ -1742,7 +1736,7 @@ CONST char *file, *src;
 			if (!Xstrchr("ynaYNA", c)) continue;
 			key = Xtoupper(c);
 			VOID_C Xfprintf(Xstderr, "%c%s", c, c_left);
-			Xfflush(Xstderr);
+			VOID_C Xfflush(Xstderr);
 		}
 		VOID_C fputnl(Xstderr);
 		if (key == 'N') return(0);
@@ -2145,8 +2139,8 @@ char *CONST argv[];
 	putterms(T_CLEAR);
 	tflush();
 #else
-	Xfputs(t_clear, Xstdout);
-	Xfflush(Xstdout);
+	puts2(t_clear);
+	VOID_C Xfflush(Xstdout);
 #endif
 	VOID_C fputnl(Xstderr);
 
