@@ -11,10 +11,7 @@
 #include "roman.h"
 #include "func.h"
 #include "kanji.h"
-
-#ifdef	DEP_PTY
 #include "termemu.h"
-#endif
 
 #define	KN_HIRA		0
 #define	KN_KATA		1
@@ -233,8 +230,8 @@ int type, kana;
 			tmp[1] = ((buf[1] & 0xf0) >> 4);
 			if (tmp[0] >= 10 || tmp[1] >= 10) return((u_int)0);
 			buf[1] = tmp[1] * 10 + tmp[0];
-			buf[0] += 0xa0;
-			buf[1] += 0xa0;
+			buf[0] += (char)0xa0;
+			buf[1] += (char)0xa0;
 			buf[2] = type = '\0';
 			break;
 #ifdef	DEP_UNICODE
@@ -283,13 +280,11 @@ u_int c;
 }
 #endif	/* DEP_PTY */
 
-static VOID NEAR imeputch(c, so)
-int c, so;
+static VOID NEAR imeputch(c, isstandout)
+int c, isstandout;
 {
-	if (so) Xputterm(T_STANDOUT);
-	VOID_C XXputch(c);
+	VOID_C Xattrprintf("%c", isstandout, c);
 	if (ime_xposp) (*ime_xposp)++;
-	if (so) Xputterm(END_STANDOUT);
 }
 
 static int NEAR imeputs(s)
@@ -656,12 +651,12 @@ int type;
 	kanjierrno = 0;
 }
 
-static VOID NEAR imeputcursor(xpos, n, so)
-int xpos, n, so;
+static VOID NEAR imeputcursor(xpos, n, isstandout)
+int xpos, n, isstandout;
 {
 	(*ime_locate)(xpos, ime_line);
 	n += (n < 10) ? '0' : 'A' - 10;
-	imeputch(n, so);
+	imeputch(n, isstandout);
 #ifdef	DEP_PTY
 	if (ptylist[win].pid) /*EMPTY*/;
 	else
@@ -803,6 +798,7 @@ int sig;
 				if (plen < 0) return(-1);
 				break;
 			default:
+				if (ch >= K_MIN) break;
 				if (Xisdigit(ch)) ch -= '0';
 				else if (Xisupper(ch) && Xisxdigit(ch))
 					ch -= 'A' - 10;
@@ -980,6 +976,7 @@ int sig, type;
 					ch = K_ESC;
 				break;
 			default:
+				if (ch >= K_MIN) break;
 				if (Xisdigit(ch)) ch -= '0';
 				else if (Xisupper(ch) && Xisxdigit(ch))
 					ch -= 'A' - 10;
