@@ -21,10 +21,8 @@
 
 #include "unixemu.h"
 #include "kconv.h"
-
-#ifdef	DEP_PTY
 #include "termemu.h"
-#endif
+
 #if	MSDOS
 #include <io.h>
 #endif
@@ -105,14 +103,16 @@ static VOID NEAR sj2j __P_((char *, CONST u_char *));
 static VOID NEAR j2sj __P_((char *, CONST u_char *));
 #endif
 #ifdef	DEP_UNICODE
+# ifndef	DEP_EMBEDUNITBL
 static int NEAR openunitbl __P_((CONST char *));
 static u_char *NEAR newunitbl __P_((ALLOC_T));
+# endif
 #define	getword(s, n)		(((u_short)((s)[(n) + 1]) << 8) | (s)[n])
-#define	skread(f,o,s,n)		(Xlseek(f, o, L_SET) >= (off_t)0 \
+#define	skread(f, o, s, n)	(Xlseek(f, o, L_SET) >= (off_t)0 \
 				&& sureread(f, s, n) == n)
 #endif
 #ifndef	_NOKANJICONV
-# ifdef	DEP_UNICODE
+# if	defined (DEP_UNICODE) && !defined (DEP_EMBEDUNITBL)
 static int NEAR opennftbl __P_((CONST char *, int, u_int *));
 # endif
 static int NEAR toenglish __P_((char *, CONST u_char *, int));
@@ -126,7 +126,7 @@ static VOID NEAR fromucs2 __P_((char *, int *, u_int));
 static int NEAR toutf8 __P_((char *, CONST u_char *, int));
 static int NEAR fromutf8 __P_((char *, CONST u_char *, int));
 static int NEAR toutf8nf __P_((char *, CONST u_char *, int, int));
-static int NEAR fromutf8nf __P_((char *, CONST u_char *, int, int));
+static int NEAR fromutf8nf __P_((char *, CONST u_char *, int, int, int));
 # endif	/* DEP_UNICODE */
 static int NEAR bin2hex __P_((char *, int));
 static int NEAR tohex __P_((char *, CONST u_char *, int));
@@ -193,8 +193,13 @@ static CONST langtable langlist[] = {
 #define	LANGLISTSIZ		arraysize(langlist)
 #endif	/* !_NOKANJICONV || (!_NOENGMES && !_NOJPNMES) */
 #ifdef	DEP_UNICODE
+# ifdef	DEP_EMBEDUNITBL
+extern CONST u_char unitblbuf[];
+extern u_int unitblent;
+# else
 static u_char *unitblbuf = NULL;
 static u_int unitblent = 0;
+# endif
 static CONST kconv_t rsjistable[] = {
 	{0x8470, 0x8440, 0x0f},		/* Cyrillic small letters */
 	{0x8480, 0x844f, 0x12},		/* Why they converted ? */
@@ -355,11 +360,18 @@ static CONST u_char j2sjtable3[128] = {
 #endif	/* !_NOKANJICONV || (DEP_DOSEMU && CODEEUC) */
 #ifndef	_NOKANJICONV
 # ifdef	DEP_UNICODE
-static u_int nftblnum = 0;
-static u_int nflen = 0;
+#  ifdef	DEP_EMBEDUNITBL
+extern int nftblnum;
+extern int nflen;
+extern CONST u_char *nftblbuf[];
+extern u_int nftblent[];
+#  else
+static int nftblnum = 0;
+static int nflen = 0;
 static u_char **nftblbuf = NULL;
 static u_int *nftblent = 0;
-# endif
+#  endif
+# endif	/* DEP_UNICODE */
 static CONST u_char hctypetable[256] = {
 	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	/* 0x00 */
 	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	/* 0x10 */
@@ -402,7 +414,7 @@ static CONST u_char h2btable[256] = {
 	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0	/* 0xf0 */
 };
 #endif	/* !_NOKANJICONV */
-#ifndef	_NOKANJIFCONV
+#ifdef	DEP_FILECONV
 static CONST kpathtable kpathlist[] = {
 	{&sjispath, SJIS},
 	{&eucpath, EUC},
@@ -420,7 +432,7 @@ static CONST kpathtable kpathlist[] = {
 	{&noconvpath, NOCNV},
 };
 #define	KPATHLISTSIZ		arraysize(kpathlist)
-#endif	/* !_NOKANJIFCONV */
+#endif	/* DEP_FILECONV */
 
 
 int onkanji1(s, ptr)
@@ -624,6 +636,7 @@ int max;
 #endif	/* !_NOKANJICONV || (DEP_DOSEMU && CODEEUC) */
 
 #ifdef	DEP_UNICODE
+# ifndef	DEP_EMBEDUNITBL
 static int NEAR openunitbl(file)
 CONST char *file;
 {
@@ -642,9 +655,9 @@ CONST char *file;
 	if (!unitblpath || !*unitblpath) Xstrcpy(path, file);
 	else strcatdelim2(path, unitblpath, file);
 
-# ifdef	FD
+#  ifdef	FD
 	noconv++;
-# endif
+#  endif
 	fd = -1;
 	if ((fd = Xopen(path, O_BINARY | O_RDONLY, 0666)) < 0) fd = -1;
 	else if (unitblent) /*EMPTY*/;
@@ -653,9 +666,9 @@ CONST char *file;
 		VOID_C Xclose(fd);
 		fd = -1;
 	}
-# ifdef	FD
+#  ifdef	FD
 	noconv--;
-# endif
+#  endif
 
 	return(fd);
 }
@@ -676,22 +689,21 @@ ALLOC_T size;
 VOID readunitable(nf)
 int nf;
 {
-# ifndef	_NOKANJICONV
+#  ifndef	_NOKANJICONV
 	u_char buf[2], **tblbuf;
 	u_int *tblent;
 	int i;
-# endif
+#  endif
 	u_char *tbl;
 	ALLOC_T size;
 	int fd;
 
-	if (unitblbuf) {
-# ifndef	_NOKANJICONV
-		if (nf && !nftblbuf) /*EMPTY*/;
-		else
-# endif
-		return;
-	}
+	if (!unitblbuf) /*EMPTY*/;
+#  ifndef	_NOKANJICONV
+	else if (nf && !nftblbuf) /*EMPTY*/;
+#  endif
+	else return;
+
 	if ((fd = openunitbl(UNICODETBL)) < 0) return;
 	size = (ALLOC_T)unitblent * 4;
 
@@ -705,7 +717,7 @@ int nf;
 		unitblbuf = tbl;
 	}
 
-# ifndef	_NOKANJICONV
+#  ifndef	_NOKANJICONV
 	if (nf && !nftblbuf) {
 		if (!skread(fd, (off_t)size + 2, buf, 2)) {
 			VOID_C openunitbl(NULL);
@@ -753,31 +765,30 @@ int nf;
 		nftblbuf = tblbuf;
 		nftblent = tblent;
 	}
-# endif	/* !_NOKANJICONV */
+#  endif	/* !_NOKANJICONV */
 
 	VOID_C openunitbl(NULL);
 }
 
 VOID discardunitable(VOID_A)
 {
-# ifndef	_NOKANJICONV
+#  ifndef	_NOKANJICONV
 	int i;
-# endif
+#  endif
 
-	if (unitblbuf) {
-		free(unitblbuf);
-		unitblbuf = NULL;
-	}
-# ifndef	_NOKANJICONV
+	if (unitblbuf) free(unitblbuf);
+	unitblbuf = NULL;
+#  ifndef	_NOKANJICONV
 	if (nftblbuf) {
 		for (i = 0; i < nftblnum; i++) free(nftblbuf[i]);
 		free(nftblbuf);
-		nftblbuf = NULL;
 	}
+	nftblbuf = NULL;
 	if (nftblent) free(nftblent);
 	nftblent = NULL;
-# endif	/* !_NOKANJICONV */
+#  endif	/* !_NOKANJICONV */
 }
+# endif	/* !DEP_EMBEDUNITBL */
 
 u_int unifysjis(wc, russ)
 u_int wc;
@@ -808,14 +819,19 @@ u_int cnvunicode(wc, encode)
 u_int wc;
 int encode;
 {
-	u_char *cp, buf[4];
-	u_int r, w, ofs, min, max;
+# ifndef	DEP_EMBEDUNITBL
 	int fd;
+# endif
+	CONST u_char *cp;
+	u_char buf[4];
+	u_int r, w, ofs, min, max;
 
 	wc &= 0xffff;
 	if (encode < 0) {
+# ifndef	DEP_EMBEDUNITBL
 		VOID_C openunitbl(NULL);
 		if (!unicodebuffer) discardunitable();
+# endif
 		return(0);
 	}
 
@@ -860,10 +876,15 @@ int encode;
 		if (wc == 0x309a) return(0x814b);
 	}
 
+# ifndef	DEP_EMBEDUNITBL
 	if (unicodebuffer && !unitblbuf) readunitable(0);
+# endif
 	cp = buf;
 	ofs = min = max = 0;
-	if (unitblbuf) {
+# ifndef	DEP_EMBEDUNITBL
+	if (unitblbuf)
+# endif
+	{
 		if (encode) {
 			cp = unitblbuf;
 			for (ofs = 0; ofs < unitblent; ofs++) {
@@ -886,11 +907,12 @@ int encode;
 			}
 		}
 	}
+# ifndef	DEP_EMBEDUNITBL
 	else if ((fd = openunitbl(UNICODETBL)) < 0) ofs = unitblent;
 	else if (encode) {
 		if (Xlseek(fd, (off_t)2, L_SET) < (off_t)0) ofs = unitblent;
 		else for (ofs = 0; ofs < unitblent; ofs++) {
-			if (sureread(fd, cp, 4) != 4) {
+			if (sureread(fd, buf, 4) != 4) {
 				ofs = unitblent;
 				break;
 			}
@@ -904,7 +926,7 @@ int encode;
 		for (;;) {
 			ofs = (min + max) / 2;
 			if (ofs <= min || ofs >= max) break;
-			if (!skread(fd, (off_t)(ofs - 1) * 4 + 2, cp, 4)) {
+			if (!skread(fd, (off_t)(ofs - 1) * 4 + 2, buf, 4)) {
 				ofs = min = max = 0;
 				break;
 			}
@@ -914,6 +936,7 @@ int encode;
 			else break;
 		}
 	}
+# endif	/* !DEP_EMBEDUNITBL */
 
 	if (encode) {
 		if (ofs < unitblent) r = getword(cp, 0);
@@ -929,7 +952,7 @@ int encode;
 #endif	/* DEP_UNICODE */
 
 #ifndef	_NOKANJICONV
-# ifdef	DEP_UNICODE
+# if	defined (DEP_UNICODE) && !defined (DEP_EMBEDUNITBL)
 static int NEAR opennftbl(file, nf, entp)
 CONST char *file;
 int nf;
@@ -959,7 +982,7 @@ u_int *entp;
 
 	return(fd);
 }
-# endif	/* DEP_UNICODE */
+# endif	/* DEP_UNICODE && !DEP_EMBEDUNITBL */
 
 static int NEAR toenglish(buf, s, max)
 char *buf;
@@ -1266,22 +1289,33 @@ int max, knj, asc, io;
 }
 
 # ifdef	DEP_UNICODE
-VOID ucs2normalization(buf, ptrp, max, wc, nf)
+int ucs2normalization(buf, ptrp, max, wc, nf)
 u_short *buf;
 int *ptrp, max;
 u_int wc;
 int nf;
 {
-	u_char *cp, *new;
+# ifndef	DEP_EMBEDUNITBL
+	int fd;
+# endif
+	CONST u_char *cp;
+	u_char *new;
 	u_int w, ofs, ent;
-	int n, fd;
+	int n;
 
 	cp = new = NULL;
 	ofs = ent = (u_int)0;
+# ifndef	DEP_EMBEDUNITBL
 	if (unicodebuffer && !nftblbuf) readunitable(1);
+# endif
 
 	if (wc < MINUNICODE || wc > MAXUNICODE) /*EMPTY*/;
-	else if (nftblbuf && nf <= nftblnum) {
+# ifdef	DEP_EMBEDUNITBL
+	else if (nf <= nftblnum)
+# else
+	else if (nftblbuf && nf <= nftblnum)
+# endif
+	{
 		n = 2 + nflen * 2;
 		cp = nftblbuf[nf - 1];
 		ent = nftblent[nf - 1];
@@ -1291,13 +1325,14 @@ int nf;
 			cp += n;
 		}
 	}
+# ifndef	DEP_EMBEDUNITBL
 	else if ((fd = opennftbl(UNICODETBL, nf, &ent)) < 0) ent = (u_int)0;
 	else {
 		n = 2 + nflen * 2;
 		cp = new = (u_char *)malloc(n);
 		if (!new) ent = (u_int)0;
 		else for (ofs = 0; ofs < ent; ofs++) {
-			if (sureread(fd, cp, n) != n) {
+			if (sureread(fd, new, n) != n) {
 				ofs = ent;
 				break;
 			}
@@ -1305,33 +1340,51 @@ int nf;
 			if (wc == w) break;
 		}
 	}
+# endif	/* !DEP_EMBEDUNITBL */
 
-	if (ofs >= ent) buf[(*ptrp)++] = wc;
-	else for (n = 0; n < nflen && *ptrp < max; n++) {
+	n = (ofs < ent) ? 0 : -1;
+	if (!buf) /*EMPTY*/;
+	else if (n < 0) buf[(*ptrp)++] = wc;
+	else while (n < nflen && *ptrp < max) {
 		cp += 2;
 		w = getword(cp, 0);
 		if (!w) break;
 		buf[(*ptrp)++] = w;
+		n++;
 	}
 	if (new) free(new);
+
+	return(n);
 }
 
-u_int ucs2denormalization(buf, ptrp, nf)
+u_int ucs2denormalization(buf, ptrp, nf, io)
 CONST u_short *buf;
-int *ptrp, nf;
+int *ptrp, nf, io;
 {
-	u_char *cp, *new;
+# ifndef	DEP_EMBEDUNITBL
 	off_t top;
-	u_int w, ofs, min, max, ent;
-	int i, j, n, fd;
+	u_int ent;
+	int fd;
+# endif
+	CONST u_char *cp;
+	u_char *new;
+	u_int w, ofs, min, max;
+	int i, j, n;
 
 	cp = new = NULL;
 	ofs = min = max = (u_int)0;
 	i = 0;
+# ifndef	DEP_EMBEDUNITBL
 	if (unicodebuffer && !nftblbuf) readunitable(1);
+# endif
 
 	if (buf[*ptrp] < MINUNICODE || buf[*ptrp] > MAXUNICODE) /*EMPTY*/;
-	else if (nftblbuf && nf <= nftblnum) {
+# ifdef	DEP_EMBEDUNITBL
+	else if (nf <= nftblnum)
+# else
+	else if (nftblbuf && nf <= nftblnum)
+# endif
+	{
 		n = 2 + nflen * 2;
 		max = nftblent[nf - 1] + 1;
 		for (;;) {
@@ -1349,6 +1402,7 @@ int *ptrp, nf;
 			if (!w) break;
 		}
 	}
+# ifndef	DEP_EMBEDUNITBL
 	else if ((fd = opennftbl(UNICODETBL, nf, &ent)) < 0
 	|| (top = Xlseek(fd, (off_t)0, L_INCR)) < (off_t)0)
 		/*EMPTY*/;
@@ -1360,7 +1414,7 @@ int *ptrp, nf;
 		else for (;;) {
 			ofs = (min + max) / 2;
 			if (ofs <= min || ofs >= max) break;
-			if (!skread(fd, (off_t)(ofs - 1) * n + top, cp, n)) {
+			if (!skread(fd, (off_t)(ofs - 1) * n + top, new, n)) {
 				ofs = min = max = (u_int)0;
 				break;
 			}
@@ -1375,8 +1429,14 @@ int *ptrp, nf;
 			if (!w) break;
 		}
 	}
+# endif	/* !DEP_EMBEDUNITBL */
 
-	if (ofs <= min || ofs >= max) w = buf[(*ptrp)++];
+	if (ofs <= min || ofs >= max) {
+		w = buf[(*ptrp)++];
+		if (io != L_FNAME) /*EMPTY*/;
+		else if (ucs2normalization(NULL, NULL, 0, w, nf) >= 0)
+			kanjierrno = UTF8;
+	}
 	else {
 		w = getword(cp, 0);
 		*ptrp += i;
@@ -1547,7 +1607,7 @@ int max, nf;
 	for (i = j = 0; s[i] && j < max; j++) u1[j] = toucs2(s, &i);
 	u1[j] = (u_short)0;
 	for (i = j = 0; u1[i] && j < max; i++)
-		ucs2normalization(u2, &j, max, u1[i], nf);
+		VOID_C ucs2normalization(u2, &j, max, u1[i], nf);
 	u2[j] = (u_short)0;
 	for (i = j = 0; u2[i] && j < max; i++) {
 		len = 1;
@@ -1565,10 +1625,10 @@ int max, nf;
 	return(j);
 }
 
-static int NEAR fromutf8nf(buf, s, max, nf)
+static int NEAR fromutf8nf(buf, s, max, nf, io)
 char *buf;
 CONST u_char *s;
-int max, nf;
+int max, nf, io;
 {
 	u_short *u1, *u2;
 	int i, j;
@@ -1583,7 +1643,7 @@ int max, nf;
 	for (i = j = 0; s[i] && j < max; j++) u1[j] = ucs2fromutf8(s, &i);
 	u1[j] = (u_short)0;
 	for (i = j = 0; u1[i] && j < max; j++)
-		u2[j] = ucs2denormalization(u1, &i, nf);
+		u2[j] = ucs2denormalization(u1, &i, nf, io);
 	u2[j] = (u_short)0;
 	for (i = j = 0; u2[i] && j + 1 < max; i++) fromucs2(buf, &j, u2[i]);
 
@@ -1871,7 +1931,7 @@ int max, in, out, *lenp, io;
 				case M_UTF8:
 				case I_UTF8:
 					*lenp = fromutf8nf(buf, (u_char *)s,
-						max, in - UTF8);
+						max, in - UTF8, io);
 					break;
 # endif
 				case HEX:
@@ -1950,7 +2010,7 @@ int in, out, io;
 {
 	char *buf;
 
-	buf = newkanjiconv(*sp, in, out, in);
+	buf = newkanjiconv(*sp, in, out, io);
 	if (buf != *sp) {
 		free(*sp);
 		*sp = buf;
@@ -2046,12 +2106,11 @@ int drv;
 }
 
 /*ARGSUSED*/
-char *convput(buf, path, needfile, rdlink, rrreal, codep)
+char *convput(buf, path, rrreal, flags)
 char *buf;
 CONST char *path;
-int needfile, rdlink;
 char *rrreal;
-int *codep;
+int flags;
 {
 # if	defined (DEP_FILECONV) || defined (DEP_DOSEMU)
 	char kbuf[MAXPATHLEN];
@@ -2070,7 +2129,6 @@ int *codep;
 	int n;
 
 	if (rrreal) *rrreal = '\0';
-	if (codep) *codep = NOCNV;
 
 	if (noconv || isdotdir(path)) {
 # ifdef	DEP_DOSEMU
@@ -2088,7 +2146,8 @@ int *codep;
 # ifdef	DEP_URLPATH
 	url = _urlpath(path, NULL, NULL);
 # endif
-	if (needfile && strdelim(path, 0)) needfile = 0;
+	if ((flags & CVP_NEEDFILE) && strdelim(path, 0))
+		flags &= ~CVP_NEEDFILE;
 	if (norealpath) cp = path;
 	else {
 		if ((file = strrdelim(path, 0))) {
@@ -2118,7 +2177,7 @@ int *codep;
 			file = path;
 			copycurpath(rpath);
 		}
-		Xrealpath(rpath, rpath, RLP_READLINK);
+		VOID_C Xrealpath(rpath, rpath, RLP_READLINK);
 		tmp = strcatdelim(rpath);
 		Xstrncpy(tmp, file, MAXPATHLEN - 1 - (tmp - rpath));
 		cp = rpath;
@@ -2127,7 +2186,6 @@ int *codep;
 # ifdef	DEP_DOSEMU
 	if ((n = dospath(cp, kbuf))) {
 		cp = kbuf;
-		if (codep) *codep = SJIS;
 #  ifdef	DEP_FILECONV
 		fputok = 0;
 #  endif
@@ -2140,15 +2198,16 @@ int *codep;
 		else
 #  endif
 		c = getkcode(cp);
-		if (codep) *codep = c;
 		cp = kanjiconv2(kbuf, cp, MAXPATHLEN - 1, DEFCODE, c, L_FNAME);
 	}
 # endif	/* DEP_FILECONV */
 # ifdef	DEP_ROCKRIDGE
 	if (!norockridge && (cp = detranspath(cp, rbuf)) == rbuf) {
 		if (rrreal) Xstrcpy(rrreal, rbuf);
-		if (rdlink && rrreadlink(cp, buf, MAXPATHLEN - 1) >= 0) {
-			if (needfile && strdelim(buf, 0)) needfile = 0;
+		if (!(flags & CVP_READLINK)) /*EMPTY*/;
+		else if (rrreadlink(cp, buf, MAXPATHLEN - 1) >= 0) {
+			if ((flags & CVP_NEEDFILE) && strdelim(buf, 0))
+				flags &= ~CVP_NEEDFILE;
 			if (*buf == _SC_ || !(tmp = strrdelim(rbuf, 0)))
 				cp = buf;
 			else {
@@ -2156,13 +2215,13 @@ int *codep;
 				Xstrncpy(tmp, buf, MAXPATHLEN - (tmp - rbuf));
 				cp = rbuf;
 			}
-			Xrealpath(cp, rpath, RLP_READLINK);
+			VOID_C Xrealpath(cp, rpath, RLP_READLINK);
 			cp = detranspath(rpath, rbuf);
 		}
 	}
 # endif
 	if (cp == path) return((char *)path);
-	if (needfile && (file = strrdelim(cp, 0))) file++;
+	if ((flags & CVP_NEEDFILE) && (file = strrdelim(cp, 0))) file++;
 	else file = cp;
 
 # ifdef	DEP_DOSEMU
