@@ -14,13 +14,9 @@
 #include "func.h"
 #include "kanji.h"
 #include "version.h"
-
-#ifdef	DEP_ORIGSHELL
 #include "system.h"
-#endif
-#ifdef	DEP_PTY
 #include "termemu.h"
-#endif
+
 #ifdef	DEP_IME
 #include "roman.h"
 #endif
@@ -60,6 +56,11 @@ extern u_int _stklen = 0x5800;
 #define	harderr_t		int
 #endif
 
+typedef struct _sigfunc_t {
+	int sig;
+	int (*func)__P_((VOID_A));
+} sigfunc_t;
+
 #if	MSDOS
 # ifndef	BSPATHDELIM
 extern char *adjustpname __P_((char *));
@@ -231,6 +232,57 @@ static int timersec = 0;
 #ifdef	SIGWINCH
 static int nowinch = 0;
 #endif
+static CONST sigfunc_t sigfunclist[] = {
+#ifdef	SIGINT
+	{SIGINT, ignore_int},
+#endif
+#ifdef	SIGQUIT
+	{SIGQUIT, ignore_quit},
+#endif
+#ifdef	SIGCONT
+	{SIGCONT, ignore_cont},
+#endif
+#ifdef	SIGHUP
+	{SIGHUP, hanguperror},
+#endif
+#ifdef	SIGILL
+	{SIGILL, illerror},
+#endif
+#ifdef	SIGTRAP
+	{SIGTRAP, traperror},
+#endif
+#ifdef	SIGIOT
+	{SIGIOT, ioerror},
+#endif
+#ifdef	SIGEMT
+	{SIGEMT, emuerror},
+#endif
+#ifdef	SIGFPE
+	{SIGFPE, floaterror},
+#endif
+#ifdef	SIGBUS
+	{SIGBUS, buserror},
+#endif
+#ifdef	SIGSEGV
+	{SIGSEGV, segerror},
+#endif
+#ifdef	SIGSYS
+	{SIGSYS, syserror},
+#endif
+#ifdef	SIGPIPE
+	{SIGPIPE, pipeerror},
+#endif
+#ifdef	SIGTERM
+	{SIGTERM, terminate},
+#endif
+#ifdef	SIGXCPU
+	{SIGXCPU, xcpuerror},
+#endif
+#ifdef	SIGXFSZ
+	{SIGXFSZ, xsizerror},
+#endif
+	{-1, NULL}
+};
 
 
 #if	MSDOS && !defined (PROTECTED_MODE)
@@ -250,10 +302,10 @@ CONST char *s;
 	static int doing = 0;
 	int duperrno;
 
-	if (doing == 1) exit2(2);
-	if (doing == 2) exit(2);
+	if (doing > 1) exit(2);
+	if (doing) exit2(2);
 	duperrno = errno;
-	doing = 1;
+	doing++;
 
 	sigvecset(0);
 	if (isorgpid()) {
@@ -269,15 +321,12 @@ CONST char *s;
 	}
 
 	if (!s) s = progname;
-	if (dumbterm <= 2) Xfputc('\007', Xstderr);
+	if (dumbterm <= 2) VOID_C Xfputc('\007', Xstderr);
 	errno = duperrno;
 	if (errno) perror2(s);
-	else {
-		Xfputs(s, Xstderr);
-		VOID_C fputnl(Xstderr);
-	}
+	else errputs(s, 1);
 	VOID_C Xfclose(Xstderr);
-	doing = 2;
+	doing++;
 
 	if (isorgpid()) {
 		inittty(1);
@@ -316,7 +365,7 @@ int n;
 static VOID NEAR signalexit(sig)
 int sig;
 {
-	signal2(sig, SIG_IGN);
+	VOID_C signal2(sig, SIG_IGN);
 
 	if (isorgpid()) {
 		printf_urgent++;
@@ -352,7 +401,7 @@ int sig;
 		printf_urgent--;
 	}
 
-	signal2(sig, SIG_DFL);
+	VOID_C signal2(sig, SIG_DFL);
 	VOID_C kill(getpid(), sig);
 }
 
@@ -362,7 +411,7 @@ static int ignore_alrm(VOID_A)
 	int duperrno;
 
 	duperrno = errno;
-	signal2(SIGALRM, (sigcst_t)ignore_alrm);
+	VOID_C signal2(SIGALRM, (sigcst_t)ignore_alrm);
 	errno = duperrno;
 
 	return(0);
@@ -375,7 +424,7 @@ static int ignore_winch(VOID_A)
 	int duperrno;
 
 	duperrno = errno;
-	signal2(SIGWINCH, (sigcst_t)ignore_winch);
+	VOID_C signal2(SIGWINCH, (sigcst_t)ignore_winch);
 	errno = duperrno;
 
 	return(0);
@@ -388,7 +437,7 @@ static int ignore_int(VOID_A)
 	int duperrno;
 
 	duperrno = errno;
-	signal2(SIGINT, (sigcst_t)ignore_int);
+	VOID_C signal2(SIGINT, (sigcst_t)ignore_int);
 	errno = duperrno;
 
 	return(0);
@@ -401,7 +450,7 @@ static int ignore_quit(VOID_A)
 	int duperrno;
 
 	duperrno = errno;
-	signal2(SIGQUIT, (sigcst_t)ignore_quit);
+	VOID_C signal2(SIGQUIT, (sigcst_t)ignore_quit);
 	errno = duperrno;
 
 	return(0);
@@ -414,7 +463,7 @@ static int ignore_cont(VOID_A)
 	int duperrno;
 
 	duperrno = errno;
-	signal2(SIGCONT, (sigcst_t)ignore_cont);
+	VOID_C signal2(SIGCONT, (sigcst_t)ignore_cont);
 # if	!MSDOS
 	suspended = 1;
 #  ifdef	DEP_ORIGSHELL
@@ -630,7 +679,7 @@ int xmax, ymax;
 	for (i = 0;; i++) {
 		if (!(cp = getwsize(xmax, ymax))) {
 			if (n_line >= row) break;
-			cp = NOROW_K;
+			cp = "Line size is too small for operation.";
 		}
 		if (!i) {
 			Xputterms(T_CLEAR);
@@ -638,18 +687,15 @@ int xmax, ymax;
 			keyflush();
 		}
 
-		if (i & 1) XXcputs(SCRSZ_K);
-		else {
-			Xputterm(T_STANDOUT);
-			XXcputs(cp);
-			Xputterm(END_STANDOUT);
-		}
+		if (i & 1)
+			cp = "Terminate with the ESC key, or widen a screen.";
+		Xattrputs(cp, !(i & 1));
 		Xputterm(T_BELL);
 		Xcputnl();
 		Xtflush();
 		if (kbhit2(1000000L) && getkey3(0, inputkcode, 0) == K_ESC) {
 			errno = 0;
-			error(INTR_K);
+			error("Interrupted by the ESC key.");
 		}
 	}
 	dumbterm = dupdumbterm;
@@ -690,7 +736,12 @@ int forced;
 		x = n_column;
 		y = n_line;
 		checkscreen(-1, -1);
-		if (isorgpid()) {
+
+# ifdef	DEP_ORIGSHELL
+		if (shellmode || !isorgpid()) /*EMPTY*/;
+		else
+# endif
+		{
 			if (x != n_column || y != n_line) rewritefile(1);
 			if (subwindow) VOID_C ungetkey2(K_CTRL('L'), 1);
 		}
@@ -699,12 +750,13 @@ int forced;
 
 static int wintr(VOID_A)
 {
+	sigblk_t old;
 	int duperrno;
 
 	duperrno = errno;
-	signal2(SIGWINCH, SIG_IGN);
+	suspendsignal(SIGWINCH, (sigcst_t)wintr, &old);
 	pollscreen(1);
-	signal2(SIGWINCH, (sigcst_t)wintr);
+	resumesignal(SIGWINCH, &old);
 	errno = duperrno;
 
 	return(0);
@@ -758,10 +810,11 @@ int hide;
 #ifdef	SIGALRM
 static int trapalrm(VOID_A)
 {
+	sigblk_t old;
 	int duperrno;
 
 	duperrno = errno;
-	signal2(SIGALRM, SIG_IGN);
+	suspendsignal(SIGALRM, (sigcst_t)trapalrm, &old);
 # if	defined (DEP_ORIGSHELL) && defined (DEP_PTY)
 	if (isshptymode()) {
 		if (mypid == shellpid) printtime(hideclock);
@@ -772,7 +825,7 @@ static int trapalrm(VOID_A)
 # ifdef	SIGWINCH
 	pollscreen(0);
 # endif
-	signal2(SIGALRM, (sigcst_t)trapalrm);
+	resumesignal(SIGALRM, &old);
 	errno = duperrno;
 
 	return(0);
@@ -803,14 +856,14 @@ int set;
 	if (set == old) /*EMPTY*/;
 	else if (set) {
 #ifdef	SIGALRM
-		signal2(SIGALRM, (sigcst_t)trapalrm);
+		VOID_C signal2(SIGALRM, (sigcst_t)trapalrm);
 		noalrm = 0;
 #endif
 #ifdef	SIGTSTP
-		signal2(SIGTSTP, SIG_IGN);
+		VOID_C signal2(SIGTSTP, SIG_IGN);
 #endif
 #ifdef	SIGWINCH
-		signal2(SIGWINCH, (sigcst_t)wintr);
+		VOID_C signal2(SIGWINCH, (sigcst_t)wintr);
 #endif
 #ifdef	DEP_DOSDRIVE
 		doswaitfunc = waitmes;
@@ -826,14 +879,14 @@ int set;
 	}
 	else {
 #ifdef	SIGALRM
-		signal2(SIGALRM, (sigcst_t)ignore_alrm);
+		VOID_C signal2(SIGALRM, (sigcst_t)ignore_alrm);
 		noalrm = 1;
 #endif
 #ifdef	SIGTSTP
-		signal2(SIGTSTP, SIG_DFL);
+		VOID_C signal2(SIGTSTP, SIG_DFL);
 #endif
 #ifdef	SIGWINCH
-		signal2(SIGWINCH, (sigcst_t)ignore_winch);
+		VOID_C signal2(SIGWINCH, (sigcst_t)ignore_winch);
 #endif
 #ifdef	DEP_DOSDRIVE
 		doswaitfunc = NULL;
@@ -886,7 +939,7 @@ VOID title(VOID_A)
 		VOID_C XXputch('#');
 		n++;
 	}
-	cp = (iswellomit()) ? nullstr : " (c)1995-2010 T.Shirai  ";
+	cp = (iswellomit()) ? nullstr : " (c)1995-2019 T.Shirai  ";
 	XXcputs(cp);
 	n = n_column - len - strlen2(cp) - n;
 	while (n-- > 0) VOID_C XXputch(' ');
@@ -1001,7 +1054,7 @@ CONST char *line;
 	else i = execpseudoshell(cp, F_IGNORELIST | F_NOCOMLINE);
 	if (i) {
 		Xputterm(L_CLEAR);
-		VOID_C XXcprintf("%s, line %d: %s", file, n, ILFNC_K);
+		VOID_C XXcprintf("%s, line %d: %k", file, n, ILFNC_K);
 		Xcputnl();
 		Xputterm(L_CLEAR);
 		VOID_C XXcprintf("\t%s", line);
@@ -1024,7 +1077,7 @@ int exist;
 # endif
 	XFILE *fp;
 	char *cp, *tmp, *fold, *line;
-	int cont;
+	int cont, siz;
 #endif	/* !DEP_ORIGSHELL */
 	int n, er;
 
@@ -1082,8 +1135,9 @@ int exist;
 
 		if (!fold) fold = line;
 		else if (*line) {
-			fold = Xrealloc(fold, strlen(fold) + strlen(line) + 1);
-			strcat(fold, line);
+			siz = strlen(fold) + strlen(line);
+			fold = Xrealloc(fold, siz + 1);
+			VOID_C Xstrlcat(fold, line, siz);
 		}
 
 		if (cont) {
@@ -1239,8 +1293,8 @@ char *CONST envp[];
 	origpath = Xstrdup(buf);
 	if ((cp = searchenv(ENVPWD, envp))) {
 		*fullpath = '\0';
-		Xrealpath(cp, fullpath, 0);
-		Xrealpath(fullpath, buf, RLP_READLINK);
+		VOID_C Xrealpath(cp, fullpath, 0);
+		VOID_C Xrealpath(fullpath, buf, RLP_READLINK);
 		if (!strpathcmp(origpath, buf)) {
 			Xfree(origpath);
 			origpath = Xstrdup(fullpath);
@@ -1258,7 +1312,7 @@ char *CONST envp[];
 	else
 #endif
 	{
-		Xrealpath(cp, buf, RLP_READLINK);
+		VOID_C Xrealpath(cp, buf, RLP_READLINK);
 		Xfree(tmp);
 		if ((tmp = strrdelim(buf, 0))) *tmp = '\0';
 		progpath = Xstrdup(buf);
@@ -1284,7 +1338,7 @@ char *CONST *argv;
 		}
 #endif	/* !MSDOS */
 		for (i = 0; i < 2; i++) loadhistory(i);
-		entryhist(origpath, HST_PATH | HST_UNIQ);
+		VOID_C entryhist(origpath, HST_PATH | HST_UNIQ);
 	}
 #ifdef	DEP_LOGGING
 	startlog(argv);
@@ -1368,7 +1422,7 @@ int status;
 # endif
 	chdir2(NULL);
 # if	!defined (_NOUSEHASH) && !defined (DEP_ORIGSHELL)
-	searchhash(NULL, NULL, NULL);
+	VOID_C searchhash(NULL, NULL, NULL);
 # endif
 # ifndef	_NOROCKRIDGE
 	detranspath(NULL, NULL);
@@ -1411,56 +1465,13 @@ char *CONST argv[], *CONST envp[];
 	_harderr(criticalerror);
 #endif
 
-#ifdef	SIGHUP
-	signal2(SIGHUP, (sigcst_t)hanguperror);
-#endif
-#ifdef	SIGINT
-	signal2(SIGINT, (sigcst_t)ignore_int);
-#endif
-#ifdef	SIGQUIT
-	signal2(SIGQUIT, (sigcst_t)ignore_quit);
-#endif
-#ifdef	SIGCONT
-	signal2(SIGCONT, (sigcst_t)ignore_cont);
-#endif
-#ifdef	SIGILL
-	signal2(SIGILL, (sigcst_t)illerror);
-#endif
-#ifdef	SIGTRAP
-	signal2(SIGTRAP, (sigcst_t)traperror);
-#endif
-#ifdef	SIGIOT
-	signal2(SIGIOT, (sigcst_t)ioerror);
-#endif
-#ifdef	SIGEMT
-	signal2(SIGEMT, (sigcst_t)emuerror);
-#endif
-#ifdef	SIGFPE
-	signal2(SIGFPE, (sigcst_t)floaterror);
-#endif
-#ifdef	SIGBUS
-	signal2(SIGBUS, (sigcst_t)buserror);
-#endif
-#ifdef	SIGSEGV
-	signal2(SIGSEGV, (sigcst_t)segerror);
-#endif
-#ifdef	SIGSYS
-	signal2(SIGSYS, (sigcst_t)syserror);
-#endif
-#ifdef	SIGPIPE
-	signal2(SIGPIPE, (sigcst_t)pipeerror);
-#endif
-#ifdef	SIGTERM
-	signal2(SIGTERM, (sigcst_t)terminate);
-#endif
-#ifdef	SIGXCPU
-	signal2(SIGXCPU, (sigcst_t)xcpuerror);
-#endif
-#ifdef	SIGXFSZ
-	signal2(SIGXFSZ, (sigcst_t)xsizerror);
-#endif
+	for (i = 0; i < arraysize(sigfunclist); i++) {
+		VOID_C signal2(sigfunclist[i].sig,
+			(sigcst_t)(sigfunclist[i].func));
+	}
 	sigvecset(0);
 
+	VOID_C entryhist(NULL, HST_INIT);
 #ifdef	DEP_DYNAMICLIST
 	helpindex = copystrarray(NULL, orighelpindex, NULL, MAXHELPINDEX);
 	for (i = 0; origbindlist[i].key >= 0; i++) /*EMPTY*/;
@@ -1536,7 +1547,7 @@ char *CONST argv[], *CONST envp[];
 	}
 	interactive = fdmode = 1;
 	setshellvar(envp);
-	prepareterm();
+	if (prepareterm() < 0) error(NULL);
 	argc = initoption(argc, argv);
 	if (dumbterm > 1) {
 		errno = 0;
