@@ -10,13 +10,8 @@
 #include "kconv.h"
 #include "func.h"
 #include "kanji.h"
-
-#ifdef	DEP_ORIGSHELL
 #include "system.h"
-#endif
-#ifdef	DEP_PTY
 #include "termemu.h"
-#endif
 
 typedef struct _localetable {
 	CONST char *env;
@@ -24,19 +19,21 @@ typedef struct _localetable {
 	char *org;
 } localetable;
 
+#define	HISTTYPES		2
 #if	0
 #define	MACROMETA		((char)-1)
 #endif
-#if	defined (USESYSCONF) && defined (_SC_ARG_MAX)
-#define	MAXARGNUM		sysconf(_SC_ARG_MAX)
-#else
-# ifdef	ARG_MAX
-# define	MAXARGNUM	ARG_MAX
+#ifdef	ARG_MAX
+#define	MAXARGNUM		ARG_MAX
+#else	/* !ARG_MAX */
+# ifdef	NCARGS
+# define	MAXARGNUM	NCARGS
 # else
-#  ifdef	NCARGS
-#  define	MAXARGNUM	NCARGS
-#  endif
+# define	MAXARGNUM	(20 * 1024)
 # endif
+#endif	/* !ARG_MAX */
+#if	defined (_NOEXTRAMACRO) && !defined (MAXCOMMSTR)
+#define	MAXCOMMSTR		1023
 #endif
 
 #ifdef	DEP_DOSLFN
@@ -81,6 +78,7 @@ static char *NEAR evalalias __P_((CONST char *));
 #else
 #define	system3			system2
 #endif
+static int NEAR newhistory __P_((int));
 
 #ifdef	DEP_ORIGSHELL
 CONST char *promptstr2 = NULL;
@@ -91,14 +89,15 @@ userfunctable userfunclist[MAXFUNCTABLE];
 int maxuserfunc = 0;
 #endif
 CONST char *promptstr = NULL;
-char **history[2] = {NULL, NULL};
-char *histfile[2] = {NULL, NULL};
-short histsize[2] = {0, 0};
-short histno[2] = {0, 0};
-short savehist[2] = {0, 0};
+char **history[HISTTYPES];
+char *histfile[HISTTYPES];
+short histno[HISTTYPES];
+short histumask[HISTTYPES];
+short histsize[HISTTYPES];
+short savehist[HISTTYPES];
 int n_args = 0;
 
-static short histbufsize[2] = {0, 0};
+static short histbufsize[HISTTYPES];
 static localetable localelist[] = {
 	{"LC_COLLATE", "C", NULL},
 	{"LC_CTYPE", "", NULL},
@@ -150,7 +149,7 @@ int code;
 	if (code < 0) {
 		cp = _evalpath(&((*bufp)[ptr]), &((*bufp)[eol]),
 			EA_NOEVALQ | EA_NOUNIQDELIM);
-		Xrealpath(cp, rpath, RLP_READLINK);
+		VOID_C Xrealpath(cp, rpath, RLP_READLINK);
 		Xfree(cp);
 		code = getkcode(rpath);
 	}
@@ -227,7 +226,7 @@ int flags;
 			arg = path;
 		else
 #endif
-		arg = convput(conv, arg, 1, 0, NULL, NULL);
+		arg = convput(conv, arg, NULL, CVP_NEEDFILE);
 	}
 	else {
 		strcatdelim2(path, dir, arg);
@@ -357,11 +356,8 @@ int blen, mlen, eol;
 macrostat *stp;
 int flags;
 {
-# ifdef	MAXCOMMSTR
-	int m, f, optr;
-# endif
 	char *cp, *s, *dir, *tmp;
-	int i, n, len, flen, rlen;
+	int i, n, m, f, len, flen, rlen, optr;
 
 # ifdef	_NOARCHIVE
 	dir = NULL;
@@ -385,25 +381,19 @@ int flags;
 		len = setarg(bufp, ptr, sizep,
 			dir, filelist[filepos].name, flags);
 		ptr += len;
-# ifdef	MAXCOMMSTR
 		if (ptr + flen > MAXCOMMSTR) {
 			Xfree(tmp);
 			return(-1);
 		}
-# endif
 		*bufp = c_realloc(*bufp, ptr + flen + 1, sizep);
 		memcpy(&((*bufp)[ptr]), cp, flen);
 		ptr += flen;
 	}
 	else for (i = n = 0; i < maxfile; i++) {
 		if (!isarg(&(filelist[i]))) continue;
-# ifdef	MAXCOMMSTR
 		optr = ptr;
-# endif
 		if (n) {
-# ifdef	MAXCOMMSTR
 			if (ptr + blen + 1 > MAXCOMMSTR) break;
-# endif
 			*bufp = c_realloc(*bufp, ptr + blen + 1 + 1, sizep);
 			(*bufp)[ptr++] = ' ';
 			memcpy(&((*bufp)[ptr]), tmp, blen);
@@ -411,7 +401,6 @@ int flags;
 		}
 		len = setarg(bufp, ptr, sizep, dir, filelist[i].name, flags);
 		ptr += len;
-# ifdef	MAXCOMMSTR
 		if (ptr + flen > MAXCOMMSTR) {
 			if (!n) {
 				Xfree(tmp);
@@ -420,7 +409,6 @@ int flags;
 			ptr = optr;
 			break;
 		}
-# endif
 		*bufp = c_realloc(*bufp, ptr + flen + 1, sizep);
 		memcpy(&((*bufp)[ptr]), cp, flen);
 		ptr += flen;
@@ -429,14 +417,13 @@ int flags;
 		n++;
 	}
 
-# ifdef	MAXCOMMSTR
 	if (ptr + rlen > MAXCOMMSTR) {
 		cp = &(s[MAXCOMMSTR - ptr]);
 		len = rlen - (MAXCOMMSTR - ptr);
 		for (i = 0; i < len; i++) {
-#  ifndef	MACROMETA
+# ifndef	MACROMETA
 			if (skipquote(cp, &i)) continue;
-#  endif
+# endif
 			if ((m = isneedargs(cp, i, &f))) {
 				if (f & F_MARK) (stp -> needmark)--;
 				i += m - 1;
@@ -444,7 +431,6 @@ int flags;
 		}
 		rlen = MAXCOMMSTR - ptr;
 	}
-# endif	/* MAXCOMMSTR */
 	*bufp = c_realloc(*bufp, ptr + rlen + 1, sizep);
 	memcpy(&((*bufp)[ptr]), s, rlen);
 	ptr += rlen;
@@ -470,7 +456,7 @@ int needmark;
 	org = arg;
 	alias[0] = '\0';
 # endif
-	arg = convput(conv, arg, 1, 0, NULL, NULL);
+	arg = convput(conv, arg, NULL, CVP_NEEDFILE);
 # ifdef	FAKEUNINIT
 	m = 0;	/* fake for -Wuninitialized */
 # endif
@@ -485,11 +471,8 @@ int needmark;
 	}
 	if (!(format[j])) return(Xstrdup(format));
 	buf = c_realloc(NULL, 0, &size);
-# ifdef	MAXCOMMSTR
 	if (j > MAXCOMMSTR) i = j = 0;
-	else
-# endif
-	{
+	else {
 		buf = c_realloc(buf, j + 1, &size);
 		memcpy(buf, format, j);
 		src = &(format[j + m]);
@@ -511,9 +494,7 @@ int needmark;
 			if ((flags & F_NOEXT) && (cp = Xstrrchr(ins, '.')))
 				len = cp - ins;
 			else len = strlen(ins);
-# ifdef	MAXCOMMSTR
 			if (j + len > MAXCOMMSTR) break;
-# endif
 			buf = c_realloc(buf, j + len + 1, &size);
 			Xstrncpy(&(buf[j]), ins, len);
 			Xfree(new);
@@ -527,9 +508,7 @@ int needmark;
 					len += m - 1;
 				}
 			}
-# ifdef	MAXCOMMSTR
 			if (j + len > MAXCOMMSTR) break;
-# endif
 			buf = c_realloc(buf, j + len + 1, &size);
 			memcpy(&(buf[j]), src, len);
 			j += len;
@@ -716,7 +695,7 @@ macrostat *stp;
 		cnvptr = j;
 #endif
 	}
-#if	defined (_NOEXTRAMACRO) && defined (MAXCOMMSTR)
+#ifdef	_NOEXTRAMACRO
 	if (j > MAXCOMMSTR) {
 		if (isttyiomode) warning(E2BIG, command);
 		else {
@@ -773,7 +752,7 @@ macrostat *stp;
 #endif	/* _NOEXTRAMACRO */
 
 	if (!(flags & F_ARGSET) && arg && *arg) {
-		arg = convput(conv, arg, 1, 1, NULL, NULL);
+		arg = convput(conv, arg, NULL, CVP_NEEDFILE | CVP_READLINK);
 		if (checksc(NULL, 0, arg) < 0) cp = NULL;
 		else {
 			cp = Xmalloc(strlen(arg) + 2 + 1);
@@ -786,7 +765,7 @@ macrostat *stp;
 		Xfree(cp);
 
 		len = strlen(arg);
-#if	defined (_NOEXTRAMACRO) && defined (MAXCOMMSTR)
+#ifdef	_NOEXTRAMACRO
 		if (j + len + 1 > MAXCOMMSTR) /*EMPTY*/;
 		else
 #endif
@@ -839,10 +818,13 @@ int iscomm;
 	static int lastptr = 0;
 	char *cp, *arg, *buf, *dir, **argv, **argv2;
 	ALLOC_T size;
-	int i, j, n, m, argc, argc2, len, maxarg, min, next, ret, hit, flags;
+	int i, j, n, m, argc, argc2, len, min, next, ret, hit, flags;
+	int argmax, arglen;
+
+	if (!argvp) {
+		if (argcp) *argcp = lastptr;
+		else lastptr = iscomm;
 
-	if (!argcp || !argvp) {
-		lastptr = 0;
 		return(-1);
 	}
 
@@ -858,43 +840,35 @@ int iscomm;
 	dir = (archivefile) ? archivedir : NULL;
 # endif
 
-# ifdef	MAXCOMMSTR
-	maxarg = 0;
-# endif
+	arglen = 0;
 	for (n = i = 0; n < argc; n++) {
-# ifdef	MAXCOMMSTR
-		if (n) maxarg++;
-# endif
 		for (len = 0; argv[n][len]; len++) {
 # ifndef	MACROMETA
 			if ((m = skipquote(argv[n], &len))) {
-#  ifdef	MAXCOMMSTR
-				maxarg += m;
-#  endif
+				arglen += m;
 				continue;
 			}
-# endif	/* !MACROMETA */
+# endif
 			if ((m = isneedargs(argv[n], len, &flags))) {
 				if (flags) i++;
 				len += m - 1;
 			}
-# ifdef	MAXCOMMSTR
-			else maxarg++;
-# endif
+			else arglen++;
 		}
+		arglen++;
 	}
 	if (!i) return(ret);
 
 # ifdef	MAXCOMMSTR
-	if (iscomm > 0 && maxarg > MAXCOMMSTR) return(seterrno(E2BIG));
+	argmax = MAXCOMMSTR + 1;
 # else	/* !MAXCOMMSTR */
-#  ifdef	MAXARGNUM
-	maxarg = MAXARGNUM - 2048;
-	if (maxarg > 20 * 1024) maxarg = 20 * 1024;
-	maxarg -= countvar(env);
-	if (iscomm > 0 && argc > maxarg) return(seterrno(E2BIG));
+#  if	defined (USESYSCONF) && defined (_SC_ARG_MAX)
+	if ((argmax = sysconf(_SC_ARG_MAX)) > 0) argmax -= 2048;
+	else
 #  endif
+	argmax = MAXARGNUM - 2048;
 # endif	/* !MAXCOMMSTR */
+	if (iscomm > 0 && arglen > argmax) return(seterrno(E2BIG));
 
 	argc2 = 0;
 	argv2 = NULL;
@@ -938,14 +912,12 @@ int iscomm;
 				cp = (iscomm < 0 || n) ? arg : NULL;
 
 				len = setarg(&buf, j, &size, dir, cp, flags);
-# ifdef	MAXCOMMSTR
-				maxarg += len;
-				if (iscomm > 0 && maxarg > MAXCOMMSTR) {
+				arglen += len;
+				if (iscomm > 0 && arglen > argmax) {
 					Xfree(buf);
 					freevar(argv2);
 					return(seterrno(E2BIG));
 				}
-# endif
 				j += len;
 			}
 		}
@@ -956,12 +928,6 @@ int iscomm;
 		if (min < 0 || !hit || (iscomm >= 0 && !n)) continue;
 
 		while (next < maxfile) {
-# if	!defined (MAXCOMMSTR) && defined (MAXARGNUM)
-			if (iscomm > 0 && argc + (argc2 - n) + 1 > maxarg) {
-				if (flags & F_REMAIN) ret++;
-				break;
-			}
-# endif
 			arg = filelist[next].name;
 			for (i = j = 0; argv[n][i]; i++) {
 # ifndef	MACROMETA
@@ -992,20 +958,16 @@ int iscomm;
 
 					len = setarg(&buf, j, &size,
 						dir, arg, flags);
-# ifdef	MAXCOMMSTR
-					maxarg += len;
-					if (iscomm > 0 && maxarg > MAXCOMMSTR)
+					arglen += len;
+					if (iscomm > 0 && arglen > argmax)
 						break;
-# endif
 					j += len;
 				}
 			}
-# ifdef	MAXCOMMSTR
 			if (argv[n][i]) {
 				if (flags & F_REMAIN) ret++;
 				break;
 			}
-# endif
 			buf[j++] = '\0';
 			argv2 = (char **)Xrealloc(argv2,
 				(argc2 + 2) * sizeof(char *));
@@ -1026,10 +988,10 @@ int iscomm;
 	}
 
 	min = -1;
-	for (next = lastptr; next < maxfile; next++)
-	if (isarg(&(filelist[next]))) {
-		if (min < 0) min = next;
-		else break;
+	for (next = lastptr; next < maxfile; next++) {
+		if (!isarg(&(filelist[next]))) continue;
+		if (min >= 0) break;
+		min = next;
 	}
 	arg = (min >= 0) ? filelist[min].name : filelist[filepos].name;
 
@@ -1061,14 +1023,12 @@ int iscomm;
 				if (min >= 0 && next < maxfile) ret++;
 
 				len = setarg(&buf, j, &size, NULL, cp, flags);
-# ifdef	MAXCOMMSTR
-				maxarg += len;
-				if (iscomm > 0 && maxarg > MAXCOMMSTR) {
+				arglen += len;
+				if (iscomm > 0 && arglen > argmax) {
 					Xfree(buf);
 					freevar(argv);
 					return(seterrno(E2BIG));
 				}
-# endif
 				j += len;
 			}
 		}
@@ -1160,7 +1120,7 @@ CONST char *def;
 		Xfree(cp);
 		return(NULL);
 	}
-	entryhist(tmp, HST_COMM);
+	VOID_C entryhist(tmp, HST_COMM);
 
 	return(tmp);
 }
@@ -1283,7 +1243,7 @@ macrostat *stp;
 	if (!wastty) Xstdiomode();
 	Xfree(command);
 	if (!cp) {
-		if (!wastty) VOID_C fputnl(Xstdout);
+		if (!wastty) putnl();
 		return(NULL);
 	}
 	if (!*cp) {
@@ -1345,7 +1305,7 @@ CONST char *command;
 			(stree -> comm) -> argc =
 				getsubst(i, argv, &subst, &len);
 			stripquote(argv[0], EA_STRIPQ);
-			if (argv[0] && checkinternal(argv[0]) < 0) ret = 0;
+			if (argv[0] && !getinternal(argv[0])) ret = 0;
 			freevar(subst);
 			Xfree(len);
 		}
@@ -1353,7 +1313,7 @@ CONST char *command;
 	freestree(stree);
 	Xfree(stree);
 #else	/* !DEP_ORIGSHELL */
-	if (getargs(command, &argv) > 0 && checkinternal(argv[0]) < 0) ret = 0;
+	if (getargs(command, &argv) > 0 && !getinternal(argv[0])) ret = 0;
 	freevar(argv);
 #endif	/* !DEP_ORIGSHELL */
 
@@ -1586,6 +1546,24 @@ CONST char *command;
 }
 #endif	/* !DEP_ORIGSHELL */
 
+static int NEAR newhistory(n)
+int n;
+{
+	int i, size;
+
+	if (n < 0 || n >= HISTTYPES) return(-1);
+	size = (int)histsize[n];
+	if (size > histbufsize[n]) {
+		i = (history[n]) ? histbufsize[n] + 1 : 0;
+		history[n] = (char **)Xrealloc(history[n],
+			(size + 1) * sizeof(char *));
+		while (i <= size) history[n][i++] = NULL;
+		histbufsize[n] = size;
+	}
+
+	return(size);
+}
+
 int entryhist(s, flags)
 CONST char *s;
 int flags;
@@ -1593,24 +1571,21 @@ int flags;
 	char *new;
 	int i, n, size;
 
-	n = (flags & HST_TYPE);
-	size = (int)histsize[n];
-	if (!history[n]) {
-		history[n] = (char **)Xmalloc((size + 1) * sizeof(char *));
-		for (i = 0; i <= size; i++) history[n][i] = NULL;
-		histbufsize[n] = size;
-		histno[n] = (short)0;
-	}
-	else if (size > histbufsize[n]) {
-		history[n] = (char **)Xrealloc(history[n],
-			(size + 1) * sizeof(char *));
-		for (i = histbufsize[n] + 1; i <= size; i++)
-			history[n][i] = NULL;
-		histbufsize[n] = size;
+	if (flags & HST_INIT) {
+		for (n = 0; n < HISTTYPES; n++) {
+			history[n] = NULL;
+			histfile[n] = NULL;
+			histno[n] = histumask[n] =
+			histsize[n] = savehist[n] = histbufsize[n] = (short)0;
+		}
+		return(0);
 	}
 
+	n = (flags & HST_TYPE);
+	if ((size = newhistory(n)) < 0) return(0);
+
 	if (!s || !*s) return(0);
-	new = (n == 1) ? killmeta(s) : Xstrdup(s);
+	new = (n == HST_PATH) ? killmeta(s) : Xstrdup(s);
 
 	if (histno[n]++ >= MAXHISTNO) histno[n] = (short)0;
 
@@ -1618,7 +1593,9 @@ int flags;
 		for (i = 0; i <= size; i++) {
 			if (!history[n][i]) continue;
 #if	defined (PATHNOCASE) && defined (DEP_DOSPATH)
-			if (n == 1 && *new != *(history[n][i])) continue;
+			if (n != HST_PATH) /*EMPTY*/;
+			else if (_dospath(new) != _dospath(history[n][i]))
+				continue;
 #endif
 			if (!strpathcmp(new, history[n][i])) break;
 		}
@@ -1626,8 +1603,9 @@ int flags;
 	}
 
 	Xfree(history[n][size]);
-	for (i = size; i > 0; i--) history[n][i] = history[n][i - 1];
-	*(history[n]) = new;
+	memmove((char *)&(history[n][1]), (char *)&(history[n][0]),
+		size * sizeof(char *));
+	history[n][0] = new;
 #ifdef	DEP_PTY
 	sendparent(TE_SETHISTORY, s, flags);
 #endif
@@ -1639,14 +1617,16 @@ char *removehist(n)
 int n;
 {
 	char *tmp;
-	int i, size;
+	int size;
 
+	if (n < 0 || n >= HISTTYPES) return(NULL);
 	size = (int)histsize[n];
 	if (!history[n] || size > histbufsize[n] || size <= 0) return(NULL);
 	if (--histno[n] < (short)0) histno[n] = MAXHISTNO;
-	tmp = *(history[n]);
-	for (i = 0; i < size; i++) history[n][i] = history[n][i + 1];
-	history[n][size--] = NULL;
+	tmp = history[n][0];
+	memmove((char *)&(history[n][0]), (char *)&(history[n][1]),
+		size * sizeof(char *));
+	history[n][size] = NULL;
 
 	return(tmp);
 }
@@ -1656,18 +1636,17 @@ int n;
 {
 	lockbuf_t *lck;
 	char *line;
-	int i, j, size;
+	int i, size;
 
+	if (n < 0 || n >= HISTTYPES) return(0);
 	if (!histfile[n] || !*(histfile[n])) return(0);
-	lck = lockfopen(histfile[n], "r", O_BINARY | O_RDONLY);
+	lck = lockfopen(histfile[n], "r", O_BINARY | O_RDONLY, 0);
 	if (!lck || !(lck -> fp)) {
 		lockclose(lck);
 		return(-1);
 	}
 
-	size = (int)histsize[n];
-	history[n] = (char **)Xmalloc((size + 1) * sizeof(char *));
-	histbufsize[n] = size;
+	if ((size = newhistory(n)) < 0) return(0);
 	histno[n] = (short)0;
 
 	i = -1;
@@ -1676,54 +1655,42 @@ int n;
 		if (histno[n]++ >= MAXHISTNO) histno[n] = (short)0;
 		if (i < size) i++;
 		else Xfree(history[n][i]);
-		for (j = i; j > 0; j--) history[n][j] = history[n][j - 1];
-		*(history[n]) = line;
+		memmove((char *)&(history[n][1]), (char *)&(history[n][0]),
+			i * sizeof(char *));
+		history[n][0] = line;
 	}
 	lockclose(lck);
 
-	for (i++; i <= size; i++) history[n][i] = NULL;
-
 	return(0);
 }
 
-VOID convhistory(s, fp)
-CONST char *s;
-XFILE *fp;
-{
-	char *eol;
-
-	if (!s || !*s) return;
-
-	while ((eol = Xstrchr(s, '\n'))) {
-		VOID_C Xfwrite(s, eol++ - s, fp);
-		VOID_C Xfputc('\0', fp);
-		s = eol;
-	}
-	VOID_C Xfputs(s, fp);
-	VOID_C Xfputc('\n', fp);
-}
-
 int savehistory(n)
 int n;
 {
 	lockbuf_t *lck;
 	int i, size;
 
+	if (n < 0 || n >= HISTTYPES) return(0);
 	if (!histfile[n] || !*(histfile[n]) || savehist[n] <= 0) return(0);
-	if (!history[n] || !*(history[n])) return(-1);
+	if (!history[n] || !history[n][0]) return(-1);
 	lck = lockfopen(histfile[n], "w",
-		O_BINARY | O_WRONLY | O_CREAT | O_TRUNC);
+		O_BINARY | O_WRONLY | O_CREAT | O_TRUNC, histumask[n]);
 	if (!lck || !(lck -> fp)) {
 		lockclose(lck);
 		return(-1);
 	}
 
+	Xsetflags(lck -> fp, XF_NULLCONV);
 	size = (savehist[n] > histsize[n])
 		? (int)histsize[n] : (int)savehist[n];
-	for (i = size - 1; i >= 0; i--) convhistory(history[n][i], lck -> fp);
+	for (i = size - 1; i >= 0; i--) {
+		if (!history[n][i] || !*(history[n][i])) continue;
+		if (Xfputs(history[n][i], lck -> fp) == EOF) break;
+		if (Xfputc('\n', lck -> fp) == EOF) break;
+	}
 	lockclose(lck);
 
-	return(0);
+	return((i < 0) ? 0 : -1);
 }
 
 int parsehist(str, ptrp, quote)
@@ -1832,9 +1799,11 @@ int n;
 {
 	int i;
 
-	if (!history[n] || !*(history[n])) return;
+	if (n < 0 || n >= HISTTYPES) return;
+	if (!history[n] || !history[n][0]) return;
 	for (i = 0; i <= histbufsize[n]; i++) Xfree(history[n][i]);
 	Xfree(history[n]);
+	history[n] = NULL;
 }
 #endif
 
