@@ -73,7 +73,8 @@
 #define	unixreaddir		readdir
 #define	unixrewinddir		rewinddir
 # ifdef	USEGETWD
-# define	unixgetcwd(p,s)	(char *)getwd(p)
+# define	unixgetcwd(p, s) \
+				(char *)getwd(p)
 # else
 # define	unixgetcwd	(char *)getcwd
 # endif
@@ -82,19 +83,19 @@
 #define	unixchmod(p, m)		((chmod(p, m)) ? -1 : 0)
 #define	unixutimes		rawutimes
 #define	unixunlink(p)		((unlink(p)) ? -1 : 0)
-#define	unixrename(f, t)	((rename(f,t)) ? -1 : 0)
+#define	unixrename(f, t)	((rename(f, t)) ? -1 : 0)
 #define	unixrmdir(p)		((rmdir(p)) ? -1 : 0)
 #endif	/* !MSDOS */
 #if	MSDOS && !defined (DJGPP) && !defined (FD)
 #define	unixmkdir(p, m)		((mkdir(p)) ? -1 : 0)
 #endif
 #if	!MSDOS || (defined (DJGPP) && !defined (FD))
-#define	unixmkdir(p, m)		((mkdir(p,m)) ? -1 : 0)
+#define	unixmkdir(p, m)		((mkdir(p, m)) ? -1 : 0)
 #endif
 
 #ifndef	FD
-#define	convput(b,p,n,r,rp,cp)	(p)
-#define	convget(b,p,d)		(p)
+#define	convput(b, p, rp, f)	(p)
+#define	convget(b, p, d)	(p)
 #endif
 #ifdef	CYGWIN
 #define	opendir_saw_u_cygdrive	(1 << (8 * sizeof(dirp -> __flags) - 2))
@@ -155,12 +156,10 @@ extern char fullpath[];
 static int NEAR checkpath __P_((CONST char *, char *));
 #endif
 #ifdef	CYGWIN
-static struct dirent *NEAR pseudoreaddir __P_((DIR *));
-#else
-#define	pseudoreaddir		unixreaddir
+static VOID NEAR getcygdrive __P_((VOID_A));
 #endif
 #if	defined (DEP_KANJIPATH) || defined (DEP_ROCKRIDGE) \
-||	defined (DEP_PSEUDOPATH)
+|| defined (DEP_PSEUDOPATH)
 static int NEAR getopenlist __P_((int, VOID_P));
 static VOID NEAR putopenlist __P_((int, int, VOID_P, CONST char *));
 static int NEAR chkopenlist __P_((int, VOID_P));
@@ -176,6 +175,13 @@ static char *NEAR unixgetcwd __P_((char *, ALLOC_T));
 static int NEAR unixstat __P_((CONST char *, struct stat *));
 #define	unixutimes		rawutimes
 #endif	/* MSDOS && !FD */
+#ifdef	DEP_DIRENT
+# ifdef	CYGWIN
+static struct dirent *NEAR pseudoreaddir __P_((DIR *));
+# else
+#define	pseudoreaddir		unixreaddir
+# endif
+#endif	/* DEP_DIRENT */
 #ifdef	DEP_DOSDRIVE
 static int NEAR checkchdir __P_((int, CONST char *));
 #endif
@@ -191,13 +197,9 @@ int lastdrv = -1;
 int dosdrive = 0;
 #endif
 #ifdef	DEP_URLPATH
-# ifdef	WITHNETWORK
-int urldrive = 1;
-# else
 int urldrive = 0;
-# endif
 int urlkcode = NOCNV;
-#endif	/* DEP_URLPATH */
+#endif
 #ifndef	NOSELECT
 int (*readintrfunc)__P_((VOID_A)) = NULL;
 #endif
@@ -206,10 +208,14 @@ int (*readintrfunc)__P_((VOID_A)) = NULL;
 static char cachecwd[MAXPATHLEN] = "";
 #endif
 #if	defined (DEP_KANJIPATH) || defined (DEP_ROCKRIDGE) \
-||	defined (DEP_PSEUDOPATH)
+|| defined (DEP_PSEUDOPATH)
 static openstat_t *openlist = NULL;
 static int maxopenlist = 0;
 #endif
+#ifdef	CYGWIN
+static char *cygdrive_user = NULL;
+static char *cygdrive_system = NULL;
+#endif
 
 
 int seterrno(n)
@@ -302,8 +308,10 @@ char *buf;
 {
 # if	!MSDOS
 	char *s;
+# ifdef	CODEEUC
 	int len;
-# endif
+#  endif
+# endif	/* !MSDOS */
 	CONST char *cp;
 	char tmp[MAXPATHLEN];
 	int drive;
@@ -333,19 +341,19 @@ char *buf;
 #  endif
 	unixrealpath(path, buf);
 # else	/* !MSDOS */
-# ifdef	CODEEUC
+#  ifdef	CODEEUC
 	if (!noconv || cp != path)
 		buf[ujis2sjis(buf, (u_char *)cp, sizeof(tmp) - 1)] = '\0';
 	else
-# endif
+#  endif
 	VOID_C Xsnprintf(buf, MAXPATHLEN, cp);
 	if (cp == cachecwd && *path) {
 		s = strcatdelim(buf);
+#  ifdef	CODEEUC
 		len = MAXPATHLEN - (s - buf);
-# ifdef	CODEEUC
 		if (!noconv) s[ujis2sjis(s, (u_char *)path, len - 1)] = '\0';
 		else
-# endif
+#  endif
 		VOID_C Xsnprintf(s, MAXPATHLEN - (s - buf), path);
 	}
 # endif	/* !MSDOS */
@@ -452,15 +460,51 @@ int *typep;
 		s = strcatdelim(buf);
 		VOID_C Xsnprintf(s, MAXPATHLEN - (s - buf), path);
 	}
-	Xrealpath(buf, tmp, RLP_PSEUDOPATH);
+	VOID_C Xrealpath(buf, tmp, RLP_PSEUDOPATH);
 	VOID_C Xsnprintf(buf, MAXPATHLEN, "%.*s%s", n, cp, tmp);
 
 	return(n);
 }
 #endif	/* DEP_URLPATH */
 
+#ifdef	CYGWIN
+static VOID NEAR getcygdrive(VOID_A)
+{
+	char upath[MAXPATHLEN], spath[MAXPATHLEN];
+
+	cygwin_internal(CW_GET_CYGDRIVE_PREFIXES, upath, spath);
+	if (*upath && !strpathcmp(spath, upath)) *spath = '\0';
+	cygdrive_user = Xstrdup(upath);
+	cygdrive_system = Xstrdup(spath);
+}
+
+char *getcygdrive_user(VOID_A)
+{
+	if (!cygdrive_user) getcygdrive();
+
+	return(cygdrive_user);
+}
+
+char *getcygdrive_system(VOID_A)
+{
+	if (!cygdrive_system) getcygdrive();
+
+	return(cygdrive_system);
+}
+
+# ifdef	DEBUG
+VOID freecygdrive(VOID_A)
+{
+	Xfree(cygdrive_user);
+	cygdrive_system = NULL;
+	Xfree(cygdrive_system);
+	cygdrive_user = NULL;
+}
+# endif	/* DEBUG */
+#endif	/* CYGWIN */
+
 #if	defined (DEP_KANJIPATH) || defined (DEP_ROCKRIDGE) \
-||	defined (DEP_PSEUDOPATH)
+|| defined (DEP_PSEUDOPATH)
 static int NEAR getopenlist(type, bodyp)
 int type;
 VOID_P bodyp;
@@ -608,12 +652,10 @@ char *buf;
 	char *host;
 	int type;
 # endif
-	int n, dev, drv;
+	int n, drv;
 
-	dev = DEV_NORMAL;
 # ifdef	DEP_DOSDRIVE
 	if ((n = dospath3(path))) {
-		dev = DEV_DOS;
 		drv = dosopendev(n);
 		if (drv < 0) return(-1);
 		if (drivep) *drivep = n;
@@ -623,7 +665,6 @@ char *buf;
 # endif
 # ifdef	DEP_URLPATH
 	if ((n = urlpath(path, &host, buf, &type))) {
-		dev = DEV_URL;
 		drv = urlopendev(host, type);
 		Xfree(host);
 		if (drv < 0) return(-1);
@@ -705,9 +746,9 @@ u_int mode;
 	u_short attr;
 
 	attr = (u_short)0;
-#if	MSDOS
+# if	MSDOS
 	if (mode & S_ISVTX) attr |= DS_IARCHIVE;
-#endif
+# endif
 	if (!(mode & S_IRUSR)) attr |= DS_IHIDDEN;
 	if (!(mode & S_IWUSR)) attr |= DS_IRDONLY;
 	if ((mode & S_IFMT) == S_IFDIR) attr |= DS_IFDIR;
@@ -720,7 +761,7 @@ u_int mode;
 	return(attr);
 }
 
-VOID getdostime(dp, tp, t)
+int getdostime(dp, tp, t)
 u_short *dp, *tp;
 time_t t;
 {
@@ -733,6 +774,8 @@ time_t t;
 	*tp = ((tm -> tm_hour & 0x1f) << 11)
 		+ ((tm -> tm_min & 0x3f) << 5)
 		+ ((tm -> tm_sec & 0x3e) >> 1);
+
+	return(tm -> tm_sec);
 }
 #endif	/* DEP_DOSPATH */
 
@@ -870,53 +913,60 @@ struct stat *stp;
 DIR *Xopendir(path)
 CONST char *path;
 {
-#if	defined (DEP_KANJIPATH) || defined (DEP_ROCKRIDGE) \
+# if	defined (DEP_KANJIPATH) || defined (DEP_ROCKRIDGE) \
 || defined (DEP_URLPATH) || defined (CYGWIN)
 	char tmp[MAXPATHLEN];
-#endif
-#ifdef	DEP_URLPATH
+# endif
+# if	defined (DEP_KANJIPATH) || defined (DEP_ROCKRIDGE) \
+|| defined (DEP_DOSEMU) || defined (DEP_URLPATH)
+	int dev;
+# endif
+# ifdef	DEP_URLPATH
 	char *host;
 	int n, type;
-#endif
-#ifdef	FD
+# endif
+# ifdef	FD
 	char conv[MAXPATHLEN];
-#endif
+# endif
 	DIR *dirp;
 	CONST char *cp;
-	int dev;
 
+# if	defined (DEP_KANJIPATH) || defined (DEP_ROCKRIDGE) \
+|| defined (DEP_DOSEMU) || defined (DEP_URLPATH)
 	dev = DEV_NORMAL;
-	cp = convput(conv, path, 1, 1, NULL, NULL);
-#ifdef	DEP_DOSEMU
+# endif
+	cp = convput(conv, path, NULL, CVP_NEEDFILE | CVP_READLINK);
+# ifdef	DEP_DOSEMU
 	if (_dospath(cp)) {
 		dev = DEV_DOS;
 		dirp = dosopendir(cp);
 	}
 	else
-#endif
-#ifdef	DEP_URLPATH
+# endif
+# ifdef	DEP_URLPATH
 	if ((n = urlpath(cp, &host, tmp, &type))) {
 		dev = DEV_URL;
 		dirp = urlopendir(host, type, &(tmp[n]));
 		Xfree(host);
 	}
 	else
-#endif
+# endif
 	dirp = unixopendir(cp);
 	if (!dirp) return(NULL);
 
-#if	defined (DEP_KANJIPATH) || defined (DEP_ROCKRIDGE) || defined (CYGWIN)
+# if	defined (DEP_KANJIPATH) || defined (DEP_ROCKRIDGE) || defined (CYGWIN)
 	cp = Xrealpath(path, tmp, RLP_READLINK);
-#endif
-#ifdef	CYGWIN
+#  ifdef	CYGWIN
 	if (tmp[0] != _SC_ || tmp[1])
 		dirp -> __flags |=
-			opendir_saw_u_cygdrive | opendir_saw_s_cygdrive;
-#endif
-#if	defined (DEP_KANJIPATH) || defined (DEP_ROCKRIDGE) \
-||	defined (DEP_DOSEMU) || defined (DEP_URLPATH)
-	putopenlist(OP_DIRP, dev, dirp, convput(conv, cp, 0, 1, NULL, NULL));
-#endif
+			(opendir_saw_u_cygdrive | opendir_saw_s_cygdrive);
+#  endif
+# endif	/* DEP_KANJIPATH || DEP_ROCKRIDGE || CYGWIN */
+# if	defined (DEP_KANJIPATH) || defined (DEP_ROCKRIDGE) \
+|| defined (DEP_DOSEMU) || defined (DEP_URLPATH)
+	cp = convput(conv, cp, NULL, CVP_READLINK);
+	putopenlist(OP_DIRP, dev, dirp, cp);
+# endif
 
 	return(dirp);
 }
@@ -924,58 +974,51 @@ CONST char *path;
 int Xclosedir(dirp)
 DIR *dirp;
 {
-#if	defined (DEP_KANJIPATH) || defined (DEP_ROCKRIDGE) \
-||	defined (DEP_DOSEMU) || defined (DEP_URLPATH)
+# if	defined (DEP_KANJIPATH) || defined (DEP_ROCKRIDGE) \
+|| defined (DEP_DOSEMU) || defined (DEP_URLPATH)
 	switch (delopenlist(OP_DIRP, dirp)) {
-# ifdef	DEP_DOSEMU
+#  ifdef	DEP_DOSEMU
 		case DEV_DOS:
 			return(dosclosedir(dirp));
 /*NOTREACHED*/
 			break;
-# endif
-# ifdef	DEP_URLPATH
+#  endif
+#  ifdef	DEP_URLPATH
 		case DEV_URL:
 			return(urlclosedir(dirp));
 /*NOTREACHED*/
 			break;
-# endif
+#  endif
 		default:
 			break;
 	}
-#endif	/* DEP_KANJIPATH || DEP_ROCKRIDGE || DEP_DOSEMU || DEP_URLPATH */
+# endif	/* DEP_KANJIPATH || DEP_ROCKRIDGE || DEP_DOSEMU || DEP_URLPATH */
 
 	return(unixclosedir(dirp));
 }
 
-#ifdef	CYGWIN
+# ifdef	CYGWIN
 static struct dirent *NEAR pseudoreaddir(dirp)
 DIR *dirp;
 {
 	static char *upath = NULL;
 	static char *spath = NULL;
 	struct dirent *dp;
-	char ubuf[MAXPATHLEN], sbuf[MAXPATHLEN];
 
 	if (!upath) {
-		cygwin_internal(CW_GET_CYGDRIVE_PREFIXES, ubuf, sbuf);
-		for (upath = ubuf; *upath == _SC_; upath++);
-# ifdef	DEBUG
-		_mtrace_file = "pseudoreaddir(upath)";
-# endif
-		upath = Xstrdup(upath);
-		for (spath = sbuf; *spath == _SC_; spath++);
-		if (*upath && !strpathcmp(spath, upath)) *spath = '\0';
-# ifdef	DEBUG
-		_mtrace_file = "pseudoreaddir(spath)";
-# endif
-		spath = Xstrdup(spath);
+		upath = getcygdrive_user();
+		while (*upath == _SC_) upath++;
+	}
+	if (!spath) {
+		spath = getcygdrive_system();
+		while (*spath == _SC_) spath++;
 	}
 
 	dp = readdir(dirp);
 	if (dirp -> __d_cookie != __DIRENT_COOKIE) return(dp);
 
-	if (!(*upath)) dirp -> __flags |= opendir_saw_u_cygdrive;
-	if (!(*spath)) dirp -> __flags |= opendir_saw_s_cygdrive;
+	if (!*upath) dirp -> __flags |= opendir_saw_u_cygdrive;
+	if (!*spath) dirp -> __flags |= opendir_saw_s_cygdrive;
 
 	if (dp) {
 		if (*upath && !(dirp -> __flags & opendir_saw_u_cygdrive)
@@ -1002,18 +1045,18 @@ DIR *dirp;
 
 	return(dp);
 }
-#endif	/* CYGWIN */
+# endif	/* CYGWIN */
 
 struct dirent *Xreaddir(dirp)
 DIR *dirp;
 {
-#if	!defined (DEP_DOSEMU) && !defined (DEP_URLPATH) \
+# if	!defined (DEP_DOSEMU) && !defined (DEP_URLPATH) \
 && !defined (DEP_KANJIPATH) && !defined (DEP_ROCKRIDGE)
 	return(pseudoreaddir(dirp));
-#else	/* DEP_DOSEMU || DEP_URLPATH || DEP_KANJIPATH || DEP_ROCKRIDGE */
-# if	defined (DEP_KANJIPATH) || defined (DEP_ROCKRIDGE)
+# else	/* DEP_DOSEMU || DEP_URLPATH || DEP_KANJIPATH || DEP_ROCKRIDGE */
+#  if	defined (DEP_KANJIPATH) || defined (DEP_ROCKRIDGE)
 	char path[MAXPATHLEN], conv[MAXPATHLEN];
-# endif
+#  endif
 	static st_dirent buf;
 	struct dirent *dp;
 	char *src, *dest;
@@ -1022,16 +1065,16 @@ DIR *dirp;
 	n = getopenlist(OP_DIRP, dirp);
 	dev = (n >= 0) ? openlist[n].dev : DEV_NORMAL;
 	switch (dev) {
-# ifdef	DEP_DOSEMU
+#  ifdef	DEP_DOSEMU
 		case DEV_DOS:
 			dp = (struct dirent *)dosreaddir(dirp);
 			break;
-# endif
-# ifdef	DEP_URLPATH
+#  endif
+#  ifdef	DEP_URLPATH
 		case DEV_URL:
 			dp = urlreaddir(dirp);
 			break;
-# endif
+#  endif
 		default:
 			dp = pseudoreaddir(dirp);
 			break;
@@ -1039,27 +1082,27 @@ DIR *dirp;
 	if (!dp) return(NULL);
 
 	dest = ((struct dirent *)&buf) -> d_name;
-# if	defined (CYGWIN) && defined (DEP_DOSEMU)
+#  if	defined (CYGWIN) && defined (DEP_DOSEMU)
 	/* Some versions of Cygwin have neither d_fileno nor d_ino */
 	if (dev == DEV_DOS) {
 		src = ((struct dosdirent *)dp) -> d_name;
 		wrap_reclen(&buf) = ((struct dosdirent *)dp) -> d_reclen;
 	}
 	else
-# endif
+#  endif
 	{
 		src = dp -> d_name;
 		memcpy((char *)(&buf), (char *)dp, dest - (char *)&buf);
 	}
-# if	MSDOS && defined (FD)
+#  if	MSDOS && defined (FD)
 	memcpy(&(buf.d_alias), dp -> d_alias, sizeof(dp -> d_alias));
-# endif
+#  endif
 
 	if (isdotdir(src)) {
 		Xstrcpy(dest, src);
 		return((struct dirent *)&buf);
 	}
-# if	defined (DEP_KANJIPATH) || defined (DEP_ROCKRIDGE)
+#  if	defined (DEP_KANJIPATH) || defined (DEP_ROCKRIDGE)
 	if (n >= 0) {
 		strcatdelim2(path, openlist[n].path, src);
 		if (convget(conv, path, dev) == conv) {
@@ -1069,39 +1112,39 @@ DIR *dirp;
 		Xstrcpy(dest, src);
 	}
 	else
-# endif	/* DEP_KANJIPATH || DEP_ROCKRIDGE */
-# if	defined (DEP_DOSEMU) && defined (CODEEUC)
+#  endif	/* DEP_KANJIPATH || DEP_ROCKRIDGE */
+#  if	defined (DEP_DOSEMU) && defined (CODEEUC)
 	if (dev == DEV_DOS && !noconv)
 		dest[sjis2ujis(dest, (u_char *)src, MAXNAMLEN)] = '\0';
 	else
-# endif
+#  endif
 	Xstrcpy(dest, src);
 
 	return((struct dirent *)&buf);
-#endif	/* DEP_DOSEMU || DEP_URLPATH || DEP_KANJIPATH || DEP_ROCKRIDGE */
+# endif	/* DEP_DOSEMU || DEP_URLPATH || DEP_KANJIPATH || DEP_ROCKRIDGE */
 }
 
 VOID Xrewinddir(dirp)
 DIR *dirp;
 {
-#if	defined (DEP_DOSEMU) || defined (DEP_URLPATH)
+# if	defined (DEP_DOSEMU) || defined (DEP_URLPATH)
 	switch (chkopenlist(OP_DIRP, dirp)) {
-# ifdef	DEP_DOSEMU
+#  ifdef	DEP_DOSEMU
 		case DEV_DOS:
 			dosrewinddir(dirp);
 			return;
 /*NOTREACHED*/
 			break;
-# endif
-# ifdef	DEP_URLPATH
+#  endif
+#  ifdef	DEP_URLPATH
 		case DEV_URL:
 			urlrewinddir(dirp);
 			return;
 /*NOTREACHED*/
 			break;
-# endif
+#  endif
 	}
-#endif	/* DEP_DOSEMU || DEP_URLPATH */
+# endif	/* DEP_DOSEMU || DEP_URLPATH */
 
 	unixrewinddir(dirp);
 }
@@ -1149,7 +1192,7 @@ CONST char *path;
 #endif
 	int n;
 
-	path = convput(conv, path, 1, 1, NULL, NULL);
+	path = convput(conv, path, NULL, CVP_NEEDFILE | CVP_READLINK);
 #ifndef	DEP_PSEUDOPATH
 	n = rawchdir(path);
 #else	/* DEP_PSEUDOPATH */
@@ -1301,7 +1344,7 @@ struct stat *stp;
 #endif
 	int n;
 
-	path = convput(conv, path, 1, 1, NULL, NULL);
+	path = convput(conv, path, NULL, CVP_NEEDFILE | CVP_READLINK);
 #ifdef	DEP_DOSEMU
 	if (_dospath(path)) n = dosstat(path, stp);
 	else
@@ -1335,9 +1378,9 @@ struct stat *stp;
 	int n;
 
 #ifdef	DEP_ROCKRIDGE
-	path = convput(conv, path, 1, 0, rpath, NULL);
+	path = convput(conv, path, rpath, CVP_NEEDFILE);
 #else
-	path = convput(conv, path, 1, 0, NULL, NULL);
+	path = convput(conv, path, NULL, CVP_NEEDFILE);
 #endif
 #ifdef	DEP_DOSEMU
 	if (_dospath(path)) n = doslstat(path, stp);
@@ -1380,7 +1423,7 @@ int mode;
 	CONST char *cp;
 	int n;
 
-	cp = convput(conv, path, 1, 1, NULL, NULL);
+	cp = convput(conv, path, NULL, CVP_NEEDFILE | CVP_READLINK);
 #ifdef	DEP_DOSEMU
 	if (_dospath(cp)) n = dosaccess(cp, mode);
 	else
@@ -1414,12 +1457,19 @@ int Xsymlink(name1, name2)
 CONST char *name1, *name2;
 {
 #ifdef	FD
-	char conv1[MAXPATHLEN], conv2[MAXPATHLEN];
+	char conv2[MAXPATHLEN];
+#endif
+#ifdef	DEP_KANJIPATH
+	char *cp, conv1[MAXPATHLEN];
 #endif
 	int n;
 
-	name1 = convput(conv1, name1, 1, 0, NULL, NULL);
-	name2 = convput(conv2, name2, 1, 0, NULL, NULL);
+	name2 = convput(conv2, name2, NULL, CVP_NEEDFILE);
+#ifdef	DEP_KANJIPATH
+	cp = symrealpath(name2, name1, conv1, RLP_READLINK);
+	if (cp) name1 = kanjiconv2(conv1, name1,
+		strsize(conv1), DEFCODE, getkcode(cp), L_FNAME);
+#endif
 #ifdef	DEP_DOSEMU
 	if (_dospath(name2)) n = dossymlink(name1, name2);
 	else
@@ -1443,18 +1493,20 @@ CONST char *path;
 char *buf;
 int bufsiz;
 {
+#ifdef	FD
+	char conv2[MAXPATHLEN];
+#endif
+#ifdef	DEP_KANJIPATH
+	char *cp, conv1[MAXPATHLEN];
+#endif
 #ifdef	DEP_URLPATH
 	char *host, tmp[MAXPATHLEN];
 	int type;
-#endif
-#ifdef	FD
-	char conv[MAXPATHLEN];
-	int c;
 #endif
 	char lbuf[MAXPATHLEN];
 	int n;
 
-	path = convput(conv, path, 1, 0, lbuf, &c);
+	path = convput(conv2, path, lbuf, CVP_NEEDFILE);
 #ifdef	DEP_ROCKRIDGE
 	if (*lbuf && (n = rrreadlink(lbuf, lbuf, sizeof(lbuf) - 1)) >= 0)
 		/*EMPTY*/;
@@ -1479,11 +1531,11 @@ int bufsiz;
 
 	if (n >= 0) {
 		lbuf[n] = '\0';
-#ifdef	DEP_KANJIPATH
-		path = kanjiconv2(conv, lbuf,
-			strsize(conv), c, DEFCODE, L_FNAME);
-#else
 		path = lbuf;
+#ifdef	DEP_KANJIPATH
+		cp = symrealpath(path, lbuf, conv1, RLP_READLINK);
+		if (cp) path = kanjiconv2(conv1, lbuf,
+			strsize(conv1), getkcode(cp), DEFCODE, L_FNAME);
 #endif
 		for (n = 0; n < bufsiz && path[n]; n++) buf[n] = path[n];
 	}
@@ -1504,7 +1556,7 @@ int mode;
 #endif
 	int n;
 
-	path = convput(conv, path, 1, 1, NULL, NULL);
+	path = convput(conv, path, NULL, CVP_NEEDFILE | CVP_READLINK);
 #ifdef	DEP_DOSEMU
 	if (_dospath(path)) n = doschmod(path, mode);
 	else
@@ -1546,7 +1598,7 @@ CONST struct utimes_t *utp;
 	if ((fd = open(path, O_RDONLY, 0666)) < 0) return(-1);
 	reg.x.ax = 0x5701;
 	reg.x.bx = fd;
-	getdostime(&(reg.x.dx), &(reg.x.cx), t);
+	VOID_C getdostime(&(reg.x.dx), &(reg.x.cx), t);
 	n = intcall(0x21, &reg, &sreg);
 	VOID_C close(fd);
 # else	/* !MSDOS */
@@ -1576,7 +1628,7 @@ CONST struct utimes_t *utp;
 #endif
 	int n;
 
-	path = convput(conv, path, 1, 1, NULL, NULL);
+	path = convput(conv, path, NULL, CVP_NEEDFILE | CVP_READLINK);
 #ifdef	DEP_DOSEMU
 	if (_dospath(path)) n = dosutimes(path, utp);
 	else
@@ -1602,7 +1654,7 @@ u_long flags;
 # endif
 	int n;
 
-	path = convput(conv, path, 1, 1, NULL, NULL);
+	path = convput(conv, path, NULL, CVP_NEEDFILE | CVP_READLINK);
 # ifdef	DEP_DOSEMU
 	if (_dospath(path)) n = seterrno(EACCES);
 	else
@@ -1633,7 +1685,7 @@ g_id_t gid;
 # endif
 	int n;
 
-	path = convput(conv, path, 1, 1, NULL, NULL);
+	path = convput(conv, path, NULL, CVP_NEEDFILE | CVP_READLINK);
 # ifdef	DEP_DOSEMU
 	if (_dospath(path)) n = seterrno(EACCES);
 	else
@@ -1666,7 +1718,7 @@ CONST char *path;
 #endif
 	int n;
 
-	path = convput(conv, path, 1, 1, NULL, NULL);
+	path = convput(conv, path, NULL, CVP_NEEDFILE | CVP_READLINK);
 #ifdef	DEP_DOSEMU
 	if (_dospath(path)) n = dosunlink(path);
 	else
@@ -1703,8 +1755,8 @@ CONST char *from, *to;
 #endif
 	int n;
 
-	from = convput(conv1, from, 1, 0, NULL, NULL);
-	to = convput(conv2, to, 1, 0, NULL, NULL);
+	from = convput(conv1, from, NULL, CVP_NEEDFILE);
+	to = convput(conv2, to, NULL, CVP_NEEDFILE);
 #ifdef	DEP_DOSEMU
 	if (_dospath(from)) {
 		if (_dospath(to)) n = dosrename(from, to);
@@ -1760,7 +1812,7 @@ int flags, mode;
 #ifdef	DEP_PSEUDOPATH
 	dev = DEV_NORMAL;
 #endif
-	path = convput(conv, path, 1, 1, NULL, NULL);
+	path = convput(conv, path, NULL, CVP_NEEDFILE | CVP_READLINK);
 #ifdef	DEP_DOSEMU
 	if (_dospath(path)) {
 		dev = DEV_DOS;
@@ -2053,7 +2105,7 @@ int mode;
 #endif
 	int n;
 
-	path = convput(conv, path, 1, 1, NULL, NULL);
+	path = convput(conv, path, NULL, CVP_NEEDFILE | CVP_READLINK);
 #ifdef	DEP_DOSEMU
 	if (_dospath(path)) n = dosmkdir(path, mode);
 	else
@@ -2088,7 +2140,7 @@ CONST char *path;
 #endif
 	int n;
 
-	path = convput(conv, path, 1, 1, NULL, NULL);
+	path = convput(conv, path, NULL, CVP_NEEDFILE | CVP_READLINK);
 #ifdef	DEP_DOSEMU
 	if (_dospath(path)) n = dosrmdir(path);
 	else
